package de.credo.common.bundles.license.l4j.prefpages;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.stream.Collectors;

import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.credo.common.bundles.license.l4j.Messages;
import jakarta.inject.Inject;

public class EnterLicenseDialog extends TitleAreaDialog {

	private	Text			licenseText;
	private	String			licenseKey;
	@Inject	@Translation	Messages	messages;

	@Inject
	public EnterLicenseDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	// overriding this methods allows you to set the title of the custom dialog
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(messages.enterLicenseDialogTitle);
	}

	@Override
	protected Point getInitialSize() {
		return new Point(480, 580);
	}

	@Override
    public void create() {
		super.create();

		setTitle("");
		setMessage(messages.enterLicenseDialogMessage);
    }

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());

		licenseText = new Text(container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		licenseText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		// Geht nicht. Logical Fonts scheinen in SWT nicht unterstützt zu sein. Siehe: https://stackoverflow.com/questions/221568/swt-os-agnostic-way-to-get-monospaced-font
//		font	= new Font(parent.getDisplay(), "Monospaced", parent.getFont().getFontData()[0].getHeight(), parent.getFont().getFontData()[0].getStyle());
		licenseText.setFont(JFaceResources.getTextFont());

		return container;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OPEN_ID, messages.enterLicenseDialogLoadButton, false);
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}

	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OPEN_ID) {
			FileDialog	fileDialog	= new FileDialog(getShell());
			fileDialog.setFilterPath(System.getProperty("user.home") + (System.getProperty("os.name").contains("Windows") ? "\\Desktop" : ""));
			fileDialog.setFilterExtensions(new String[] {
					"*.txt;*.l4j",
					"*.*"});
//			fileDialog.setFileName("license*.*");
			String	licenseFile	= fileDialog.open();

			if (licenseFile != null) {
				try(BufferedReader	br	= Files.newBufferedReader(Paths.get(licenseFile))) {
					licenseText.setText(br.lines().collect(Collectors.joining("\n")));
				} catch (IOException e) {
					MessageDialog.openError(getShell(), messages.enterLicenseDialogErrorMessageTitle,
								messages.enterLicenseDialogProblemWhileReadingFile
									.replace("%1", licenseFile.toString())
									.replace("%2", e.getLocalizedMessage()));
				}
			}

			// Hinüberwechseln auf den OK-Button
			getShell().setDefaultButton(getButton(IDialogConstants.OK_ID));
		}

		super.buttonPressed(buttonId);
	}

	@Override
	protected void okPressed() {
		licenseKey	= licenseText.getText();
		super.okPressed();
	}

	public String getEnteredLicenseKey() {
		return licenseKey;
	}
}
