package de.credo.common.bundles.license.l4j.prefpages;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.osgi.service.component.annotations.Component;

import com.license4j.ActivationStatus;
import com.license4j.ValidationStatus;

import de.credo.common.bundles.license.License;
import de.credo.common.bundles.license.LicenseHandler;
import de.credo.common.bundles.license.l4j.L4jLicense;
import de.credo.common.bundles.license.l4j.Messages;
import de.credo.common.bundles.preferencepages.MyPrefPage;
import jakarta.inject.Inject;

@Component
public class LicensePreferencePage extends PreferencePage implements MyPrefPage {

	private	enum ActivationMode {
		ACTIVATION,
		DEACTIVATION;
	}

	@Inject	Display				display;
	@Inject	LicenseHandler		licenseHandler;
	@Optional	@Inject			License		license;
	@Inject		@Translation	Messages	messages;

	private	DateTimeFormatter	dtf	= DateTimeFormatter.ofLocalizedDate(FormatStyle.LONG);

	private	Text	licenseIDText;
	private	Text	validationStatusText;
	private	Text	activationStatusText;
	private	Text	maintenanceExpireDateText;
	private	Text	quantityText;
	private	Button	quantityButton;

	private	Text	customerIDText;
	private	Text	registeredToText;
	private	Text	cityText;
	private	Text	countryText;

	private Button	enterButton;
	private Button	activateButton;

	public LicensePreferencePage() {
		noDefaultAndApplyButton();
	}

	@Override
	public String getTitle() {
		return messages.licensePreferencePageTitle;
	}

	@Override
	public int orderKey() {
		return 930;
	}

	@Override
	public String[] path() {
		return new String[] {messages.licensePreferencePageTitle};
	}

	@Override
	public boolean creatable() {
		return true;
	}

	@Override
	protected Control createContents(Composite parent) {
		Composite	composite	= new Composite(parent, SWT.NONE);
		GridLayout	pageLayout	= new GridLayout();
		composite.setLayout(pageLayout);

		createLicenseContentControls(composite);
		createButtonsControl(composite);

//		Label	separator	= new Label(composite, SWT.HORIZONTAL | SWT.SEPARATOR);
//		separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		updateGUI(license);

		return composite;
	}

	private void createLicenseContentControls(Composite composite) {

//		Lizenzinformationen

		Composite	licenseContentComposite	= new Composite(composite, SWT.NONE);
		licenseContentComposite.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, true));
		GridLayout	licenseContentLayout	= new GridLayout(2, false);
		licenseContentComposite.setLayout(licenseContentLayout);

		Label	serverTypeLabel	= new Label(licenseContentComposite, SWT.NONE);
		serverTypeLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		serverTypeLabel.setText(messages.licensePrefPageServerTypeLabel);
		serverTypeLabel.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));

		Label	separator	= new Label(licenseContentComposite, SWT.HORIZONTAL | SWT.SEPARATOR);
		separator.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));

		Label	licenseInfoLabel	= new Label(licenseContentComposite, SWT.NONE);
		licenseInfoLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		licenseInfoLabel.setText(messages.licensePrefPageLicenseInfoLabel);
		licenseInfoLabel.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
		new Label(licenseContentComposite, SWT.NONE);

		Label	licenseIDLabel	= new Label(licenseContentComposite, SWT.NONE);
		licenseIDLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		licenseIDLabel.setText(messages.licensePrefPageLicenseID);

		licenseIDText	= new Text(licenseContentComposite, SWT.BORDER);
		licenseIDText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		licenseIDText.setEditable(false);

		Label	validationStatusLabel	= new Label(licenseContentComposite, SWT.NONE);
		validationStatusLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		validationStatusLabel.setText(messages.licensePrefPageValidationStatus);

		validationStatusText	= new Text(licenseContentComposite, SWT.BORDER);
		validationStatusText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		validationStatusText.setEditable(false);

		Label	activationStatusLabel	= new Label(licenseContentComposite, SWT.NONE);
		activationStatusLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		activationStatusLabel.setText(messages.licensePrefPageActivationStatus);

		activationStatusText	= new Text(licenseContentComposite, SWT.BORDER);
		activationStatusText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		activationStatusText.setEditable(false);

		Label	maintenanceExpireDateLabel	= new Label(licenseContentComposite, SWT.NONE);
		maintenanceExpireDateLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		maintenanceExpireDateLabel.setText(messages.licensePrefPageMaintExpireDate);

		maintenanceExpireDateText	= new Text(licenseContentComposite, SWT.BORDER);
		maintenanceExpireDateText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		maintenanceExpireDateText.setEditable(false);

		Label	quantityLabel	= new Label(licenseContentComposite, SWT.NONE);
		quantityLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		quantityLabel.setText(messages.licensePrefPageQuantityLabel);

		Composite	quantityComposite	= new Composite(licenseContentComposite, SWT.NONE);
		quantityComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		GridLayout	quantityLayout		= new GridLayout(2, false);
		quantityLayout.marginLeft	= 0;
		quantityLayout.marginRight	= 0;
		quantityLayout.marginTop	= 0;
		quantityLayout.marginBottom	= 0;
		quantityLayout.verticalSpacing	= 0;
		quantityLayout.marginWidth	= 0;
		quantityLayout.marginHeight	= 0;
		quantityLayout.horizontalSpacing	= 20;
		quantityComposite.setLayout(quantityLayout);

		quantityText	= new Text(quantityComposite, SWT.BORDER);
		quantityText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		quantityText.setEditable(false);

		quantityButton	= new Button(quantityComposite, SWT.NONE);
		quantityButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		quantityButton.setText(messages.licensePrefPageLicenseUsageButton);
		quantityButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent arg0) {
				licenseHandler.openLicenseUsageDialog(getShell());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent arg0) {
				widgetSelected(arg0);
			}
		});

//		Kundeninformationen

		Label	customerInfoLabel	= new Label(licenseContentComposite, SWT.NONE);
		customerInfoLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		customerInfoLabel.setText(messages.licensePrefPageCustomerInfoLabel);
		customerInfoLabel.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
		new Label(licenseContentComposite, SWT.NONE);

		Label	custIDLabel	= new Label(licenseContentComposite, SWT.NONE);
		custIDLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		custIDLabel.setText(messages.licensePrefPageCustomerID);

		customerIDText	= new Text(licenseContentComposite, SWT.BORDER);
		customerIDText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		customerIDText.setEditable(false);

		Label	registeredToLabel	= new Label(licenseContentComposite, SWT.NONE);
		registeredToLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		registeredToLabel.setText(messages.licensePrefPageCustomerRegisteredTo);

		registeredToText	= new Text(licenseContentComposite, SWT.BORDER);
		registeredToText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		registeredToText.setEditable(false);

		Label	cityLabel	= new Label(licenseContentComposite, SWT.NONE);
		cityLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		cityLabel.setText(messages.licensePrefPageCustomerCity);

		cityText	= new Text(licenseContentComposite, SWT.BORDER);
		cityText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		cityText.setEditable(false);

		Label	countryLabel	= new Label(licenseContentComposite, SWT.NONE);
		countryLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
		countryLabel.setText(messages.licensePrefPageCustomerCountry);

		countryText	= new Text(licenseContentComposite, SWT.BORDER);
		countryText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		countryText.setEditable(false);
	}

	private void createButtonsControl(Composite composite) {
		Composite	buttonComposite	= new Composite(composite, SWT.NONE);
		buttonComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		FillLayout	buttonLayout	= new FillLayout();
		buttonLayout.marginWidth	= 20;
		buttonLayout.spacing		= 20;
		buttonLayout.type			= SWT.HORIZONTAL;
		buttonComposite.setLayout(buttonLayout);

		enterButton	= new Button(buttonComposite, SWT.PUSH | SWT.CENTER);
		enterButton.setText(messages.licensePrefPageEnterButton);
		enterButton.setEnabled(true);
		enterButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				licenseHandler.openEnterLicenseDialog(getShell());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		activateButton	= new Button(buttonComposite, SWT.PUSH | SWT.CENTER);
		activateButton.setText(messages.licensePrefPageActivateButton);
		activateButton.setEnabled(false);
		activateButton.setData(ActivationMode.ACTIVATION);
		activateButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				switch ((ActivationMode) ((Button) e.getSource()).getData()) {
				case ACTIVATION:
					licenseHandler.activateLicense(null);
					break;

				case DEACTIVATION:
					licenseHandler.deactivateLicense(new Object[] {Boolean.TRUE});		// "clean"

					break;
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}

		});
	}

	@Inject @Optional
	public void updateGUI(License license) {
		// Diese Routine wird wegen des Inject leider auch schon aufgerufen, wenn die Objekte selbst noch gar nicht
		// angelegt wurden. Da überholt das Inject die eigentlich vorher nötige createControls-Routine. Zumindest
		// beim ersten Mal. Ab dem zweiten Mal ist ja alles gut!
		if (licenseIDText == null || licenseIDText.isDisposed())
			return;

		// Initialisieren aller variablen Texte
		licenseIDText.setText("");
		validationStatusText.setText("");
		activationStatusText.setText("");
		maintenanceExpireDateText.setText("");
		quantityText.setText("");

		customerIDText.setText("");
		registeredToText.setText("");
		cityText.setText("");
		countryText.setText("");

		// Schneller Abbruch, wenn sowieso keine Lizenz mit kommt
		if (license == null) {
			enterButton.setEnabled(true);
			activateButton.setEnabled(false);
			quantityButton.setEnabled(false);
			return;
		}

		// Neu setzen
		if (license.isTrial()) {
			licenseIDText.setText(messages.licensePrefPageTrialLicense
					+ (license.getLicenseExpireDaysRemaining(null) < 0 ? "" : " - " + license.getLicenseExpireDaysRemaining(null) + " " + messages.licensePrefPageTrialDaysLeft));
			licenseIDText.setForeground(display.getSystemColor(SWT.COLOR_RED));

			setValid(license.isEnabled());
		} else {
			licenseIDText.setText(Long.toString(license.getLicenseId()));
		}

		switch (((L4jLicense) license).getValidationStatus()) {
		case LICENSE_MAINTENANCE_EXPIRED:
		case LICENSE_VALID:
			validationStatusText.setText(((L4jLicense) license).getValidationStatus().toString());
			validationStatusText.setForeground(display.getSystemColor(((L4jLicense) license).getValidationStatus() == ValidationStatus.LICENSE_VALID
													? SWT.COLOR_DARK_GREEN
													: SWT.COLOR_RED));

			switch (((L4jLicense) license).getActivationStatus()) {
			case ACTIVATION_NOT_REQUIRED:
			case ACTIVATION_COMPLETED:
				activationStatusText.setText(((L4jLicense) license).getActivationStatus().toString());
				activationStatusText.setForeground(display.getSystemColor(SWT.COLOR_DARK_GREEN));
				activateButton.setText(messages.licensePrefPageDeactivateButton);
				activateButton.setData(ActivationMode.DEACTIVATION);
				enterButton.setEnabled(true);
				activateButton.setEnabled(!license.isTrial() && ((L4jLicense) license).getActivationStatus() != ActivationStatus.ACTIVATION_NOT_REQUIRED);
				quantityButton.setEnabled(license.getCustomerId() > 0 && ((L4jLicense) license).getActivationStatus() != ActivationStatus.ACTIVATION_NOT_REQUIRED);
				setValid(true);
				break;

			case ALREADY_ACTIVATED_ON_ANOTHER_COMPUTER:
			case MULTIPLE_ACTIVATION_LIMIT_REACHED:
				activationStatusText.setText(((L4jLicense) license).getActivationStatus().toString());
				activationStatusText.setForeground(display.getSystemColor(SWT.COLOR_RED));
				activateButton.setText(messages.licensePrefPageDeactivateButton);
				activateButton.setData(ActivationMode.DEACTIVATION);
				enterButton.setEnabled(true);
				activateButton.setEnabled(!license.isTrial());
				quantityButton.setEnabled(license.getCustomerId() > 0);
				break;

			case ACTIVATION_REQUIRED:
			case DEACTIVATION_COMPLETED:
			case ACTIVATION_SERVER_CONNECTION_ERROR:
				activationStatusText.setText(((L4jLicense) license).getActivationStatus().toString());
				activationStatusText.setForeground(display.getSystemColor(SWT.COLOR_RED));
				activateButton.setText(messages.licensePrefPageActivateButton);
				activateButton.setData(ActivationMode.ACTIVATION);
				enterButton.setEnabled(true);
				activateButton.setEnabled(true);
				quantityButton.setEnabled(license.getCustomerId() > 0);
				if (getShell() != null) {
					getShell().setDefaultButton(activateButton);
				}
				break;

			case ACTIVATION_NOT_FOUND_ON_SERVER:
			case ACTIVATION_HARDWAREID_ERROR:
			case ACTIVATION_REJECTED_FEATURE_DISABLED:
			case LICENSE_NOT_FOUND_ON_ACTIVATION_SERVER:
			case ACTIVATION_REJECTED_IP_BLOCK_RESTRICTION:
			case DEACTIVATION_REJECTED_FEATURE_DISABLED:
			case MULTIPLE_DEACTIVATION_LIMIT_REACHED:
			default:
				activationStatusText.setText(((L4jLicense) license).getActivationStatus().toString());
				activationStatusText.setForeground(display.getSystemColor(SWT.COLOR_RED));
				enterButton.setEnabled(true);
				activateButton.setEnabled(false);
				quantityButton.setEnabled(false);
				break;
			}
			break;

		case LICENSE_EXPIRED:
		case MISMATCH_HARDWARE_ID:
		case MISMATCH_PRODUCT_EDITION:
		case MISMATCH_PRODUCT_ID:
		case MISMATCH_PRODUCT_VERSION:
		case INCORRECT_SYSTEM_TIME:
		case LICENSE_INVALID:
		case VALIDATION_REJECTED_FEATURE_DISABLED:
		case VALIDATION_REJECTED_IP_BLOCK_RESTRICTION:
		default:
			validationStatusText.setText(((L4jLicense) license).getValidationStatus().toString());
			validationStatusText.setForeground(display.getSystemColor(SWT.COLOR_RED));
			enterButton.setEnabled(true);
			activateButton.setEnabled(false);
			quantityButton.setEnabled(false);
			break;
		}

		ZonedDateTime	maintenanceExpireDate	= license.getLicenseMaintenanceExpireDate();
		if (maintenanceExpireDate != null) {
			if (maintenanceExpireDate.isAfter(ZonedDateTime.now())) {
				maintenanceExpireDateText.setText(dtf.format(maintenanceExpireDate) + " - " + license.getLicenseMaintenanceExpireDaysRemaining(ZonedDateTime.now()) + " " + messages.licensePrefPageMaintenanceDaysLeft);
			} else {
				maintenanceExpireDateText.setText(dtf.format(maintenanceExpireDate) + " - " + messages.licensePrefPageMaintenanceExpired);
				maintenanceExpireDateText.setForeground(display.getSystemColor(SWT.COLOR_DARK_YELLOW));
			}

		} else if (license.isTrial()) {
			maintenanceExpireDateText.setText("0 " + messages.licensePrefPageMaintenanceDaysLeft);
			maintenanceExpireDateText.setForeground(display.getSystemColor(SWT.COLOR_DARK_YELLOW));
		}

		quantityText.setText(Integer.toString(license.getLicenseQuantity()));

		customerIDText.setText(Integer.toString(license.getCustomerId()));
		registeredToText.setText(license.getUserRegisteredTo() != null ? license.getUserRegisteredTo() : "");
		cityText.setText(license.getUserCity() != null ? license.getUserCity() : "");
		countryText.setText(license.getUserCountry() != null ? license.getUserCountry() : "");
	}

	@Override
	public boolean performOk() {
		if (licenseIDText != null && license != null) {
			// Versuch einer automatischen Aktivierung
			if (license.isEnabled()) {
				licenseHandler.activateLicense(null);
			}

			if (!licenseHandler.checkLicense())
				return false;		// Bricht das OK ab, das Fenster bleibt geöffnet
		}

		return super.performOk();
	}

	@Inject
	private void updateLicenseHandler(LicenseHandler licenseHandler) {
		this.licenseHandler	= licenseHandler;
	}
}
