package de.credo.common.bundles.license.l4j.prefpages;

import java.net.URI;
import java.net.URL;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.credo.common.bundles.license.License;
import de.credo.common.bundles.license.LicenseHandler;
import de.credo.common.bundles.license.l4j.Messages;
import de.credo.common.xml.activations.LicenseActivationsProvider;
import de.credo.common.xml.activations.jaxb.Activation;
import de.credo.common.xml.activations.jaxb.LicenseActivations;
import de.credo.common.xml.activations.jaxb.ObjectFactory;
import jakarta.inject.Inject;

public class LicenseUsageDialog extends Dialog {

	private	final static	DateTimeFormatter	df	= DateTimeFormatter.ofLocalizedDate(FormatStyle.LONG);

	private	TableViewer		tableViewer;
	private	Table			table;

	@Inject	License			license;
	@Inject	@Translation	Messages	messages;

	public LicenseUsageDialog(Shell parentShell) {
		super(parentShell);

		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	// overriding this methods allows you to set the title of the custom dialog
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(messages.licenseUsageDialogTitle);
	}

	@Override
	protected Point getInitialSize() {
		return new Point(600, 400);
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());

		Composite tableComposite	= new Composite(container, SWT.NONE);
		tableComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		tableComposite.setLayout(new GridLayout());

		tableViewer	= new TableViewer(tableComposite, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER);
		table		= tableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		TableViewerColumn	noColumn	= createTableColumn(messages.licenseUsageDialogNumberColumn, SWT.RIGHT);
		noColumn.setLabelProvider(new StyledCellLabelProvider () {
			@Override
			public void update(ViewerCell cell) {
				Activation	element	= (Activation) cell.getElement();
				if (element != null) {
					StyledString	styledString	= getStyledString(table.getItemCount() + ".", element.isDeactivated());
					cell.setText(styledString.getString());
					cell.setStyleRanges(styledString.getStyleRanges());
					super.update(cell);
				}
			}
		});

		TableViewerColumn	idColumn	= createTableColumn(messages.licenseUsageDialogActivationIdColumn, SWT.CENTER);
		idColumn.setLabelProvider(new StyledCellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Activation	element	= (Activation) cell.getElement();
				if (element != null) {
					StyledString	styledString	= getStyledString(Long.toString(element.getID()), element.isDeactivated());
					cell.setText(styledString.getString());
					cell.setStyleRanges(styledString.getStyleRanges());
					super.update(cell);
				}
			}
		});

		TableViewerColumn	hostNameColumn	= createTableColumn(messages.licenseUsageDialogHostnameColumn, SWT.LEFT);
		hostNameColumn.setLabelProvider(new StyledCellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Activation	element	= (Activation) cell.getElement();
				if (element != null) {
					StyledString	styledString	= getStyledString(element.getHostName(), element.isDeactivated());
					cell.setText(styledString.getString());
					cell.setStyleRanges(styledString.getStyleRanges());
					super.update(cell);
				}
			}
		});

		TableViewerColumn	activationDateColumn	= createTableColumn(messages.licenseUsageDialogDateTimeColumn, SWT.LEFT);
		activationDateColumn.setLabelProvider(new StyledCellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Activation	element	= (Activation) cell.getElement();
				if (element != null) {
					StyledString	styledString	= getStyledString(df.format(element.getActivationDateTime().toGregorianCalendar().toZonedDateTime()), element.isDeactivated());
					cell.setText(styledString.getString());
					cell.setStyleRanges(styledString.getStyleRanges());
					super.update(cell);
				}
			}
		});

		TableViewerColumn	osColumn	= createTableColumn(messages.licenseUsageDialogOperatingSystemColumn, SWT.LEFT);
		osColumn.setLabelProvider(new StyledCellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Activation	element	= (Activation) cell.getElement();
				if (element != null) {
					StyledString	styledString	= getStyledString(element.getOperatingSystem(), element.isDeactivated());
					cell.setText(styledString.getString());
					cell.setStyleRanges(styledString.getStyleRanges());
					super.update(cell);
				}
			}
		});

		tableViewer.setContentProvider((IStructuredContentProvider) arg0 -> ((List<Activation>) arg0).stream()
				.toArray());

		table.addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {
			}

			@Override
			public void keyPressed(KeyEvent e) {
				if         ((e.stateMask & SWT.CTRL) != 0 && e.keyCode == 'a') {
					table.setSelection(table.getItems());
				} else if ((e.stateMask & SWT.CTRL) != 0 && (e.keyCode == 'c' || e.keyCode == SWT.INSERT)) {
					Clipboard	clipboard	= new Clipboard(Display.getDefault());
					String[]	data		= new String[] 		{convertToHtmlTable(),			convertToTabbedTable()		};
					Transfer[]	types		= new Transfer[]	{HTMLTransfer.getInstance(),	TextTransfer.getInstance()	};
					clipboard.setContents(data, types);
				}
			}
		});

		updateView();

		return container;
	}

	private Styler strikeoutStyler = new Styler() {
		@Override
		public void applyStyles(TextStyle textStyle) {
			textStyle.strikeout = true;
		}
	};

	private StyledString getStyledString(String text, Boolean deactivated) {
		return new StyledString(text != null ? text : "",
				deactivated != null && deactivated == Boolean.TRUE ? strikeoutStyler : null);
	}

	private TableViewerColumn	createTableColumn(String title, int style) {
		TableViewerColumn	column	= new TableViewerColumn(tableViewer, style);
		TableColumn			col		= column.getColumn();
		col.setText(title);
		col.setWidth(100);
		col.setResizable(true);

		return column;
	}

	private void updateView() {
		if (license.getCustomerId() > 0) {
			LicenseActivations	activations;

			try {

				URL	url	= new URI(LicenseHandler.OWN_LICENSE_SERVER_ADDRESS + "/Key/GetLicenseUsage"
						            + "?LID=" + license.getLicenseId()
						            + "&CID=" + Integer.toString(license.getCustomerId())).toURL();
				activations	= LicenseActivationsProvider.readFromUrl(url);

			} catch (Exception e) {
				ObjectFactory	factory	= new ObjectFactory();
				activations	= factory.createLicenseActivations();
				try {
					Activation	dummy	= factory.createActivation();
					dummy.setID(license.getLicenseId());
					dummy.setHostName(messages.licenseUsageDialogProblemWhileConnection);
					dummy.setActivationDateTime(DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar()));
					activations.getActivation().add(dummy);

					dummy	= factory.createActivation();
					dummy.setID(license.getLicenseId());
					dummy.setHostName(e.getCause().getCause().getMessage());
					dummy.setActivationDateTime(DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar()));
					activations.getActivation().add(dummy);
				} catch (DatatypeConfigurationException e1) {
				}
			}

			tableViewer.setInput(activations.getActivation());

			for (TableColumn tc: table.getColumns()) {
				tc.pack();
			}
		}
	}

	private String convertToTabbedTable() {
		StringBuilder	sb	= new StringBuilder();

		for (TableColumn tc: table.getColumns()) {
			if (tc != table.getColumn(0)) {
				sb.append('\t');
			}
			sb.append(tc.getText());
		}
		sb.append('\n');

		for (TableItem ti: table.getItems()) {
			for (int i = 0; i < table.getColumnCount(); i++) {
				if (i > 0) {
					sb.append('\t');
				}
				sb.append(ti.getText(i));
			}
			sb.append('\n');
		}

		return sb.toString();
	}

	private String convertToHtmlTable() {
		StringBuilder	sb	= new StringBuilder("<table>");

		sb.append("<tr>");
		for (TableColumn tc: table.getColumns()) {
			sb.append("<th>").append(tc.getText()).append("</th>");
		}
		sb.append("</tr>");

		for (TableItem ti: table.getItems()) {
			sb.append("<tr>");
			for (int i = 0; i < table.getColumnCount(); i++) {
				sb.append("<td>").append(ti.getText(i)).append("</td>");
			}
			sb.append("</tr>");
		}

		sb.append("</table>");

		return sb.toString();
	}
}
