package de.credo.common.bundles.messages;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

public class CdoLogMessage {

	private	int				windowHashCode;
	private	boolean			hasWindowHashCode;
	private	Instant			timestamp;
	private	String			origin;
	private	String			reason;
	private	Object			reference;
	private	CdoLogSeverity	severity;
	private	String			message;
	private	Throwable		throwable;

	CdoLogMessage(CdoLogSeverity severity, String message) {
		this(Instant.now(), severity, message, null);
	}

	CdoLogMessage(CdoLogSeverity severity, String message, Throwable throwable) {
		this(Instant.now(), severity, message, throwable);
	}

	CdoLogMessage(Instant timestamp, CdoLogSeverity severity, String message) {
		this(timestamp, severity, message, null);
	}

	CdoLogMessage(Instant timestamp, CdoLogSeverity severity, String message, Throwable throwable) {
		this.timestamp		= timestamp;
		this.severity		= severity;
		this.message		= message;
		this.throwable		= throwable;
	}

	public int getWindowHashCode() {
		return windowHashCode;
	}

	public void setWindowHashCode(int windowHashCode) {
		this.windowHashCode = windowHashCode;
		hasWindowHashCode	= true;
	}

	public boolean hasWindowHashCode() {
		return hasWindowHashCode;
	}

	public Instant getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Instant timestamp) {
		this.timestamp = timestamp;
	}

	public CdoLogSeverity getSeverity() {
		return severity;
	}

	public void setSeverity(CdoLogSeverity severity) {
		this.severity = severity;
	}

	public String getOrigin() {
		return origin;
	}

	public void setOrigin(String origin) {
		this.origin = origin;
	}

	public String getReason() {
		return reason;
	}

	public void setReason(String code) {
		reason = code;
	}

	public Object getReference() {
		return reference;
	}

	public void setReference(Object reference) {
		this.reference = reference;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public Throwable getThrowable() {
		return throwable;
	}

	public void setThrowable(Throwable throwable) {
		this.throwable = throwable;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (origin == null ? 0 : origin.hashCode());
		result = prime * result + (severity == null ? 0 : severity.hashCode());
		result = prime * result + (timestamp == null ? 0 : timestamp.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CdoLogMessage other = (CdoLogMessage) obj;
		if (origin == null) {
			if (other.origin != null)
				return false;
		} else if (!origin.equals(other.origin))
			return false;
		if (severity != other.severity)
			return false;
		if (timestamp == null) {
			if (other.timestamp != null)
				return false;
		} else if (!timestamp.equals(other.timestamp))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return DateTimeFormatter.ISO_TIME.format(LocalDateTime.ofInstant(getTimestamp(), ZoneId.systemDefault())) + "\t" + getSeverity() + "\t" + getOrigin() + "\t" + getReason() + "\t" + getMessage();
	}

}
