package de.credo.common.bundles.messages;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import jakarta.inject.Inject;
import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;

@Creatable
public class CdoLogSink {

	private	final static	int		MAX_MESSAGES_OF_SAME_TYPE	= 25;

	private	List<CdoLogMessage>			logMessages		= new ArrayList<>();
	private	Map<Integer, Set<Object>>	messagesCounter	= new TreeMap<>();		// Message-Hash, Reference

	private	boolean	debugPrint;

	@Inject
	@Optional
	public void addMessage(@UIEventTopic(CdoLogConstants.ADD_LOG_MESSAGE) CdoLogMessage message, @Active MTrimmedWindow activeWindow) {
		if (message != null) {
			// Schneller Abbruch, wenn diese Meldung für ein anderes Fenster ist, als das aktive
			if (message.hasWindowHashCode() && message.getWindowHashCode() != activeWindow.hashCode())
				return;

			int	mHash	= message.getMessage().hashCode();
			Set<Object>	mcEntry	= messagesCounter.get(mHash);
			if (mcEntry == null) {
				mcEntry	= new HashSet<>(5);
				messagesCounter.put(mHash, mcEntry);
			}

			if (mcEntry.add(message.getReference())) {		// nur true, wenn ein Objekt hinzugefügt werden konnte aka. noch kein Eintrag für diese Referenz existierte
				if (mcEntry.size() < MAX_MESSAGES_OF_SAME_TYPE) {
					logMessages.add(message);

					if (debugPrint) {
						System.out.println(message.toString());
					}
				}
			}
		}
	}

	public void clearMessages() {
		logMessages.clear();
		messagesCounter.clear();
	}

	public List<CdoLogMessage> getLogMessages() {
		return logMessages;
	}

	public boolean isDebugPrint() {
		return debugPrint;
	}

	public void setDebugPrint(boolean debugPrint) {
		this.debugPrint = debugPrint;
	}

}
