package de.credo.common.bundles.messages;

import java.time.Instant;

import org.eclipse.e4.core.contexts.EclipseContextFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;

public class CdoLogger {

	private	final	String	origin;

	private	IEventBroker	eventBroker;

	public CdoLogger() {
		this((String) null);
	}

	public CdoLogger(Class<?> origin) {
		this(origin.getSimpleName());
	}

	public CdoLogger(String origin) {
		if (origin != null) {
			this.origin	= origin.contains(".") ? origin.substring(origin.lastIndexOf(".") + 1) : origin;
		} else {
			this.origin	= null;
		}

		Bundle			myBundle		= FrameworkUtil.getBundle(CdoLogConstants.class);
		BundleContext	bundleContext	= myBundle.getBundleContext();
		IEclipseContext	eclipseContext	= EclipseContextFactory.getServiceContext(bundleContext);
		eventBroker	= eclipseContext.get(IEventBroker.class);
	}

	public String getOrigin() {
		return origin;
	}

	public void debug(String message) {
		post(CdoLogSeverity.DEBUG, 0, message);
	}

	public void debug(String message, String... parms) {
		post(CdoLogSeverity.DEBUG, 0, message, parms);
	}

	public void debug(String message, Throwable throwable) {
		post(CdoLogSeverity.DEBUG, 0, message, throwable);
	}

	public void debug(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.DEBUG, 0, message, throwable, parms);
	}

	public void debug(String reason, Object reference, String message) {
		post(CdoLogSeverity.DEBUG, 0, reason, reference, message);
	}

	public void debug(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.DEBUG, 0, reason, reference, message, parms);
	}

	public void debug(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.DEBUG, 0, reason, reference, message, throwable);
	}

	public void debug(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.DEBUG, 0, reason, reference, message, throwable, parms);
	}

	public void debug(int windowHashCode, String message) {
		post(CdoLogSeverity.DEBUG, windowHashCode, message);
	}

	public void debug(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.DEBUG, windowHashCode, message, parms);
	}

	public void debug(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.DEBUG, windowHashCode, message, throwable);
	}

	public void debug(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.DEBUG, windowHashCode, message, throwable, parms);
	}

	public void debug(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.DEBUG, windowHashCode, reason, reference, message);
	}

	public void debug(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.DEBUG, windowHashCode, reason, reference, message, parms);
	}

	public void debug(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.DEBUG, windowHashCode, reason, reference, message, throwable);
	}

	public void debug(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.DEBUG, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void trace(String message) {
		post(CdoLogSeverity.VERBOSE, 0, message);
	}

	public void trace(String message, String... parms) {
		post(CdoLogSeverity.VERBOSE, 0, message, parms);
	}

	public void trace(String message, Throwable throwable) {
		post(CdoLogSeverity.VERBOSE, 0, message, throwable);
	}

	public void trace(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.VERBOSE, 0, message, throwable, parms);
	}

	public void trace(String reason, Object reference, String message) {
		post(CdoLogSeverity.VERBOSE, 0, reason, reference, message);
	}

	public void trace(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.VERBOSE, 0, reason, reference, message, parms);
	}

	public void trace(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.VERBOSE, 0, reason, reference, message, throwable);
	}

	public void trace(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.VERBOSE, 0, reason, reference, message, throwable, parms);
	}

	public void trace(int windowHashCode, String message) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, message);
	}

	public void trace(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, message, parms);
	}

	public void trace(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, message, throwable);
	}

	public void trace(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, message, throwable, parms);
	}

	public void trace(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, reason, reference, message);
	}

	public void trace(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, reason, reference, message, parms);
	}

	public void trace(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, reason, reference, message, throwable);
	}

	public void trace(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.VERBOSE, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void info(String message) {
		post(CdoLogSeverity.INFO, 0, message);
	}

	public void info(String message, String... parms) {
		post(CdoLogSeverity.INFO, 0, message, parms);
	}

	public void info(String message, Throwable throwable) {
		post(CdoLogSeverity.INFO, 0, message, throwable);
	}

	public void info(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.INFO, 0, message, throwable, parms);
	}

	public void info(String reason, Object reference, String message) {
		post(CdoLogSeverity.INFO, 0, reason, reference, message);
	}

	public void info(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.INFO, 0, reason, reference, message, parms);
	}

	public void info(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.INFO, 0, reason, reference, message, throwable);
	}

	public void info(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.INFO, 0, reason, reference, message, throwable, parms);
	}

	public void info(int windowHashCode, String message) {
		post(CdoLogSeverity.INFO, windowHashCode, message);
	}

	public void info(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.INFO, windowHashCode, message, parms);
	}

	public void info(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.INFO, windowHashCode, message, throwable);
	}

	public void info(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.INFO, windowHashCode, message, throwable, parms);
	}

	public void info(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.INFO, windowHashCode, reason, reference, message);
	}

	public void info(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.INFO, windowHashCode, reason, reference, message, parms);
	}

	public void info(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.INFO, windowHashCode, reason, reference, message, throwable);
	}

	public void info(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.INFO, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void warn(String message) {
		post(CdoLogSeverity.WARNING, 0, message);
	}

	public void warn(String message, String... parms) {
		post(CdoLogSeverity.WARNING, 0, message, parms);
	}

	public void warn(String message, Throwable throwable) {
		post(CdoLogSeverity.WARNING, 0, message, throwable);
	}

	public void warn(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.WARNING, 0, message, throwable, parms);
	}

	public void warn(String reason, Object reference, String message) {
		post(CdoLogSeverity.WARNING, 0, reason, reference, message);
	}

	public void warn(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.WARNING, 0, reason, reference, message, parms);
	}

	public void warn(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.WARNING, 0, reason, reference, message, throwable);
	}

	public void warn(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.WARNING, 0, reason, reference, message, throwable, parms);
	}

	public void warn(int windowHashCode, String message) {
		post(CdoLogSeverity.WARNING, windowHashCode, message);
	}

	public void warn(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.WARNING, windowHashCode, message, parms);
	}

	public void warn(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.WARNING, windowHashCode, message, throwable);
	}

	public void warn(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.WARNING, windowHashCode, message, throwable, parms);
	}

	public void warn(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.WARNING, windowHashCode, reason, reference, message);
	}

	public void warn(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.WARNING, windowHashCode, reason, reference, message, parms);
	}

	public void warn(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.WARNING, windowHashCode, reason, reference, message, throwable);
	}

	public void warn(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.WARNING, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void error(String message) {
		post(CdoLogSeverity.ERROR, 0, message);
	}

	public void error(String message, String... parms) {
		post(CdoLogSeverity.ERROR, 0, message, parms);
	}

	public void error(String message, Throwable throwable) {
		post(CdoLogSeverity.ERROR, 0, message, throwable);
	}

	public void error(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.ERROR, 0, message, throwable, parms);
	}

	public void error(String reason, Object reference, String message) {
		post(CdoLogSeverity.ERROR, 0, reason, reference, message);
	}

	public void error(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.ERROR, 0, reason, reference, message, parms);
	}

	public void error(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.ERROR, 0, reason, reference, message, throwable);
	}

	public void error(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.ERROR, 0, reason, reference, message, throwable, parms);
	}

	public void error(int windowHashCode, String message) {
		post(CdoLogSeverity.ERROR, windowHashCode, message);
	}

	public void error(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.ERROR, windowHashCode, message, parms);
	}

	public void error(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.ERROR, windowHashCode, message, throwable);
	}

	public void error(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.ERROR, windowHashCode, message, throwable, parms);
	}

	public void error(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.ERROR, windowHashCode, reason, reference, message);
	}

	public void error(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.ERROR, windowHashCode, reason, reference, message, parms);
	}

	public void error(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.ERROR, windowHashCode, reason, reference, message, throwable);
	}

	public void error(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.ERROR, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void fatal(String message) {
		post(CdoLogSeverity.FATAL, 0, message);
	}

	public void fatal(String message, String... parms) {
		post(CdoLogSeverity.FATAL, 0, message, parms);
	}

	public void fatal(String message, Throwable throwable) {
		post(CdoLogSeverity.FATAL, 0, message, throwable);
	}

	public void fatal(String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.FATAL, 0, message, throwable, parms);
	}

	public void fatal(String reason, Object reference, String message) {
		post(CdoLogSeverity.FATAL, 0, reason, reference, message);
	}

	public void fatal(String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.FATAL, 0, reason, reference, message, parms);
	}

	public void fatal(String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.FATAL, 0, reason, reference, message, throwable);
	}

	public void fatal(String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.FATAL, 0, reason, reference, message, throwable, parms);
	}

	public void fatal(int windowHashCode, String message) {
		post(CdoLogSeverity.FATAL, windowHashCode, message);
	}

	public void fatal(int windowHashCode, String message, String... parms) {
		post(CdoLogSeverity.FATAL, windowHashCode, message, parms);
	}

	public void fatal(int windowHashCode, String message, Throwable throwable) {
		post(CdoLogSeverity.FATAL, windowHashCode, message, throwable);
	}

	public void fatal(int windowHashCode, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.FATAL, windowHashCode, message, throwable, parms);
	}

	public void fatal(int windowHashCode, String reason, Object reference, String message) {
		post(CdoLogSeverity.FATAL, windowHashCode, reason, reference, message);
	}

	public void fatal(int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(CdoLogSeverity.FATAL, windowHashCode, reason, reference, message, parms);
	}

	public void fatal(int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(CdoLogSeverity.FATAL, windowHashCode, reason, reference, message, throwable);
	}

	public void fatal(int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		post(CdoLogSeverity.FATAL, windowHashCode, reason, reference, message, throwable, parms);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String message) {
		post(severity, windowHashCode, null, null, message, (Throwable) null, (String[]) null);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String message, String... parms) {
		post(severity, windowHashCode, null, null, message, null, parms);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String message, Throwable throwable) {
		post(severity, windowHashCode, null, null, message, throwable, (String[]) null);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String message, Throwable throwable, String... parms) {
		post(severity, windowHashCode, null, null, message, throwable, parms);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String reason, Object reference, String message) {
		post(severity, windowHashCode, reason, reference, message, (Throwable) null, (String[]) null);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String reason, Object reference, String message, String... parms) {
		post(severity, windowHashCode, reason, reference, message, null, parms);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String reason, Object reference, String message, Throwable throwable) {
		post(severity, windowHashCode, reason, reference, message, (Throwable) null, (String[]) null);
	}

	public void post(CdoLogSeverity severity, int windowHashCode, String reason, Object reference, String message, Throwable throwable, String... parms) {
		if (parms != null) {
			for (int i = 0; i < parms.length; i++) {
				message	= message.replaceAll("%" + (i + 1), parms[i]);
			}
		}

		CdoLogMessage	logMessage	= new CdoLogMessage(Instant.now(), severity, message, throwable);
		logMessage.setReason(reason);
		logMessage.setReference(reference);
		logMessage.setOrigin(origin);
		if (windowHashCode != 0) {
			logMessage.setWindowHashCode(windowHashCode);
		}

		eventBroker.post(CdoLogConstants.ADD_LOG_MESSAGE, logMessage);
	}

}
