package de.credo.lookup2;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.credo.lookup2.plugins.api.L2Format;

public class FormatSelector extends TrayDialog {

	private	List<L2Format>	formats;
	private	L2Format		selection;

	private Combo	formatCombo;

	public FormatSelector(Shell parentShell, Collection<L2Format> formats, L2Format selection) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);

		this.formats	= new ArrayList<>(formats);
		this.selection	= selection;
	}

	// overriding this methods allows you to set the title of the custom dialog
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(Messages.getString("FORMAT_SELECTOR_DIALOG_TITLE"));
	}

	@Override
	protected Point getInitialSize() {
		return new Point(350, 200);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite	composite	= (Composite) super.createDialogArea(parent);
		GridLayout	layout		= new GridLayout();
		layout.marginHeight		= 10;
		layout.marginWidth		= 10;
		layout.verticalSpacing	= 10;
		composite.setLayout(layout);

		Label	formatSelectorLabel	= new Label(composite, SWT.WRAP);
		formatSelectorLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		if (formats.size() > 10) {
			formatSelectorLabel.setText(Messages.getString("FORMAT_SELECTOR_DIALOG_DESCRIPTION1"));
		} else {
			formatSelectorLabel.setText(Messages.getString("FORMAT_SELECTOR_DIALOG_DESCRIPTION2").replace("%1", Integer.toString(formats.size())));
		}

		formatCombo	= new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);
		formatCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		List<String>	formatLabels	= new ArrayList<>(formats.size() + 1);

		formatLabels.add(Messages.getString("FORMAT_SELECTOR_DIALOG_OPEN_PREFPAGE_HINT"));

		for (L2Format f: formats) {
			formatLabels.add(f.getName());
		}
		formatCombo.setItems(formatLabels.toArray(new String[0]));

		formatCombo.select(selection == null ? 0 : formatLabels.indexOf(selection.getName()));

		return composite;
	}

	public L2Format getSelection() {
		return selection;
	}

	@Override
	protected void okPressed() {
		selection	= formatCombo.getSelectionIndex() == 0 ? null : formats.get(formatCombo.getSelectionIndex() - 1);

		super.okPressed();
	}

}
