package de.credo.lookup2;

import java.util.ArrayDeque;
import java.util.Deque;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;

import de.credo.common.trees.ITreeParentNode;
import de.credo.lookup2.parsers.api.objects.L2DOMObject;
import de.credo.lookup2.parsers.api.structure.L2ParentElement;
import jakarta.inject.Inject;

@Creatable
public class History {

	private	Deque<ITreeParentNode<?, ?>>	pastElements	= new ArrayDeque<>(50);
	private	Deque<ITreeParentNode<?, ?>>	futureElements	= new ArrayDeque<>(5);
	private	ITreeParentNode<?, ?>			currentObject;

	@Inject
	public void putOnStack(@Optional L2ParentElement element) {
		if (element == null)
			return;

		if (element != currentObject) {
			if (currentObject != null) {
				pastElements.push(currentObject);
			}
			currentObject	= element;

			// Neue Elemente auf dem Haupt-Stack bedingen das Löschen der vormals(!) zukünftigen Elemente
			// (falls durch vorheriges Zurückspringen überhaupt gefüllt)
			futureElements.clear();
		}
	}

	@Inject
	public void putOnStack(@Optional L2DOMObject domObject) {
		if (domObject == null)
			return;

		if (domObject != currentObject) {
			if (currentObject != null) {
				pastElements.push(currentObject);
			}
			currentObject	= domObject;

			// Neue Elemente auf dem Haupt-Stack bedingen das Löschen der vormals(!) zukünftigen Elemente
			// (falls durch vorheriges Zurückspringen überhaupt gefüllt)
			futureElements.clear();
		}
	}

	public ITreeParentNode<?, ?> getCurrentObject() {
		return currentObject;
	}

	public boolean hasPastElements() {
		return !pastElements.isEmpty();
	}

	public boolean hasFutureElements() {
		return !futureElements.isEmpty();
	}

	public ITreeParentNode<?, ?> replayBack() {
		if (pastElements.isEmpty())
			return null;

		// Die aktuelle Auswahl auf den Stack für zukünftige Objekte packen
		futureElements.push(currentObject);

		// und Holen des letzten Elements vom Stack als current
		currentObject	= pastElements.pop();

		return currentObject;
	}

	public ITreeParentNode<?, ?> replayForth() {
		if (futureElements.isEmpty())
			return null;

		// Die aktuelle Auswahl auf den Stack für vergangene Objekte packen
		pastElements.push(currentObject);

		// und Zurückholen des vorher zukünftigen Elements vom Stack als current
		currentObject	= futureElements.pop();

		return currentObject;
	}

}
