package de.credo.lookup2;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.internal.workbench.swt.WorkbenchSWTActivator;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.workbench.UIEvents;
import org.eclipse.e4.ui.workbench.lifecycle.PostContextCreate;
import org.eclipse.e4.ui.workbench.lifecycle.PreSave;
import org.eclipse.e4.ui.workbench.lifecycle.ProcessAdditions;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.osgi.service.prefs.BackingStoreException;

import de.credo.common.bundles.cmd.CmdArgument;
import de.credo.common.bundles.cmd.CommandLine;
import de.credo.common.bundles.license.License;
import de.credo.common.bundles.license.License.LicenseTypes;
import de.credo.common.bundles.license.LicenseHandler;
import de.credo.common.bundles.license.l4j.prefpages.LicensePreferencePage;
import de.credo.common.bundles.preferencepages.MyPrefPage;
import de.credo.common.bundles.proxy.ProxyHandler;
import de.credo.common.bundles.proxy.gui.prefpages.ProxyPreferencePage;
import de.credo.common.bundles.version.BundleVersion;
import jakarta.inject.Inject;

class LifeCycle {

	private	final static	IEclipsePreferences	lifeCyclePreferences	= InstanceScope.INSTANCE.getNode(LifeCycle.class.getName());

	private	final static	String	PRODUCT_ID	= "Lookup2";

	private	final static	String	PUBLIC_KEY	=
			"""
			30820122300d06092a864886f70d01010105000382010f00303032301006\
			072a8648ce3d02002EC311215SHA512withECDSA106052b81040006031e0\
			0042ebac1339077653af6cd12ca7a133c70ed9b3d56539e5ca5670bd57eG\
			82010a0282010100aeb4f3f688cda2c9ae2fdeb7f46cf6ca795076caad90\
			87f529f6c77bb2d9cfea1da976279d4a4365f4009957360d42cefcb2b248\
			cdb399332c733e67ed6473730e93a3c8e8f523f54fddfe193bb777b1bf52\
			a09c0feb891dde36a9d603RSA4204813SHA512withRSA3c88e8738688d65\
			7cd48cc8436ce86a162933f6312c9cf3f639f5b0cf79e3946135553d1a92\
			00b9b21ac07248178d819c54abe4bf7a9e4fbfad09f79d1996c6941b6baa\
			63d1b1d0753e4097d42ccb112e333a181a2e2ada84211c51e11855ba8cf3\
			82802168ffd01b0317845b398711048f4ec79c993f98007750716d0e1832\
			4c2d8351378278c6669c83b729db039de80d8b49aa95567e07c05220a3e3\
			eec47782494350203010001
			""";
	private	final static	String	TRIAL_KEY	=
			"""
			# Lookup2 Lookup2 License (id: 1671632831075)
			9b8f006db279a505c1e74e35ca0b8000e0c027dfe5574a9d1de329698be3
			5772f78069b4938f918b9300fe471039b3d37f829b9b6968e0a356c03874
			30d9ba7c6ea626dedfcef2d640f59aa7e03e0a5ce8ab3e8167db01bfc612
			95c40fcd90f0c28ee6153c187528dca645c2494c8b8ab41ede97f2283dee
			c7fcd8e22d0d4279b074fae8e5b9ac1d5257e5f1fd09118db0d2b9b0bec6
			02c1619099eb4cce625db42115dd98b3a87934227873f873e5d9b42a8126
			85d3220ce8dacdcb0238a57803bcae7c0c0c5e42efabeb11257d7f2553e2
			7c3e705a061ff25cdc03ae8cb4b439b66146468808c5624edc0386b3d625
			738b49bc0276ed6d7ba0240005c7385dbe9ab51e47758f67018ad43f2479
			68274a0678439cce45baa2ea928fab7fb7e6abf5f422d65611c08b681aae
			3198a85f0d0eca1e4c3d29bb846bf253530e47b98c1014d7080e138d5d4e
			e0423a2a64acd2ef3e113939da82b4b09c731ac1822f6cd59994c08eafe3
			1caae8b50d28e278530a07014bd6d60a46481ad85054c560df17444956f5
			e5ad258e7dc75ca8c74cbfae2a3a7a7abed246c8c94570edb86e28048fe5
			77338a670404407913daffbba4e0c6bc6c06d750b4ace3db539f5440348a
			68f16d9967a271972f1f5b0b3ec16934b6f5595bb62deb3cec42043c51ab
			6bd2f6661ba1633d39e30db83c1a1f043e29d6d509c7409fbbd79997ec2a
			44784cfa16cb7e75aeb25e97104c9539b1f81d2153a1d2802e7daeab6513
			ef1144e060a05bcdc268d541faafbfd05e108ba65094423092a400c0cda9
			7cd0c77b93a7f22289487df34cc0a96c9642cdcd349fdfac1960a15f06f3
			a4845575ecd8135604e0661668ec0d5b79285f23acbd7f248a0fe2b6d557
			8597e177a90948ece58ab384935fb0c756799088b9bf055a51e2f4ac64ff
			be067472498145fb76461c39d4fb803638972eb04ac8a5d4a73401b3724a
			2d69abf589a6442aa2b7af02b65a93f7094a310acd7f22f2065613e10981
			323bdf22c089ceeec1447f56fc2cab880feb01eb00ee7dd5946ae78f5da4
			8588dbcdb718c64b5fca70639bc2b5de88c7e2a1ac145aecf6dab617f80d
			b7ff72945147d39c403d545d140f35ea71bce06cbde430b7298eb4ea4ee9
			c393d129d4bc
			""";

	public	final static	String	WINDOW_XPOS_NODE		= "WindowXPos";
	public	final static	String	WINDOW_YPOS_NODE		= "WindowYPos";
	public	final static	String	WINDOW_WIDTH_NODE		= "WindowWidth";
	public	final static	String	WINDOW_HEIGHT_NODE		= "WindowHeight";
	public	final static	String	DISPLAY_MONITORS_NODE	= "DisplayMonitors";
	public	final static	String	DISPLAY_BOUNDS_X_NODE	= "DisplayBoundsX";
	public	final static	String	DISPLAY_BOUNDS_Y_NODE	= "DisplayBoundsY";
	public	final static	String	DISPLAY_BOUNDS_W_NODE	= "DisplayBoundsW";
	public	final static	String	DISPLAY_BOUNDS_H_NODE	= "DisplayBoundsH";
	public	final static	String	DISPLAY_DPI_NODE		= "DisplayDPI";

	@Inject	LicenseHandler	licenseHandler;
	@Optional
	@Inject	License	license;

	@PostContextCreate
	void preInstanceCreate() {
		try {
			Location instanceLocation	= WorkbenchSWTActivator.getDefault().getInstanceLocation();
			if (instanceLocation.isLocked()) {
				if (Files.exists(Paths.get(instanceLocation.getURL().toURI()))) {
					// Schreiben aller beim Start mitgegebener Dateinamen, zum Beispiel bei einem Doppelklick
					// aus dem Windows Explorer und Autostart heraus, in eine eigene Datei, die vom Hauptthread
					// periodisch überwacht wird.
					writeParametersToStartupFile();

					// Harter Abbruch, falls es schon eine existierende Instanz gegeben hatte. Ein E4Workbench
					// .close() hilft hier nicht, weil diese vorher erst vollständig initialisiert worden muss.
					System.exit(0);
				}
			}
		} catch (IOException | URISyntaxException e) {
		}
	}

	/*
	 * Prüfung, ob dieser Anwendungsstart zusätzliche Aufrufparameter, wie etwa Dateinamen, führt.
	 * Wenn ja, werden diese in eine Hilfsdatei zur Übergabe an den bereits gestarteten ersten Prozess
	 * geschrieben, die dort regelmäßig überwacht wird. Ansonsten endet die Routine einfach ohne Aktion.
	 */
	private void writeParametersToStartupFile() {
		String[]	arguments	= Platform.getApplicationArgs();
		boolean		ignoreNext	= false;
		List<Path>	files		= new ArrayList<>();

		for (String arg: arguments) {
			// Grundsätzlich alle Parameter beginnend mit '-' und die folgenden Werte sind zu unterdrücken.
			// Achtung: Es gibt auch Parameter ohne zusätzliche Werte!
			if (arg.charAt(0) == '-' || arg.charAt(0) == '/') {
				ignoreNext	= true;
			} else {
				if (ignoreNext) {
					ignoreNext	= false;
				} else {
					java.nio.file.Path	p	= Paths.get(arg);
					if (Files.exists(p)) {
						files.add(p);
					}
				}
			}
		}

		StartupFileHandler.createAndWrite(files);
	}

	@ProcessAdditions
	void startUp(IEventBroker eventBroker, ProxyHandler proxyHandler, IEclipseContext context, IApplicationContext appContext, EModelService modelService, MApplication application, CommandLine commandLine) {
		// Ein paar Fenstermanipulationen überwachen ...
		eventBroker.subscribe(UIEvents.ElementContainer.TOPIC_SELECTEDELEMENT, new WindowCreationObserver(modelService));
		// ... ebenso wie Perspektivenwechsel Handlungen erfordern
		eventBroker.subscribe(UIEvents.ElementContainer.TOPIC_SELECTEDELEMENT, new PerspectiveChangedListener(eventBroker));
		// ... und ein paar nach dem Anwendungsstart
		eventBroker.subscribe(UIEvents.UILifeCycle.APP_STARTUP_COMPLETE, new AppStartupCompleteObserver(modelService, application, eventBroker));

		// Bundle-Version ermitteln
		BundleVersion.init(appContext.getBrandingBundle());

		// Proxy-Einstellungen laden
		proxyHandler.load();

		// Lizenzmanager initialisieren
		licenseHandler.init(licenseHandler.configBuilder()
									.setContext(context)
									.setProductIds(Map.of(
											LicenseTypes.NODE_LOCKED,	PRODUCT_ID,
											LicenseTypes.TRIAL,			TRIAL_KEY))
									.setPublicKey(PUBLIC_KEY));

		context.set(CommandLine.class, commandLine);
		commandLine.collectAllowed(() -> new CmdArgument[] {
				new CmdArgument(CommandLine.DEBUG_MODE_ARG, false, true, "Debug mode. Offers additional information (if available internally)"),
				new CmdArgument(CommandLine.SILENT_SETUP_ARG, false, true, "Silent setup only, no GUI. Requires one or more of the following parameters:")
		});
		commandLine.collectAllowed(proxyHandler);
		commandLine.collectAllowed(licenseHandler);

		commandLine.set((String[]) appContext.getArguments().get(IApplicationContext.APPLICATION_ARGS));

		if (commandLine.helpRequested()) {
			MessageDialog.openError(null, "Help!", commandLine.getHelp());
			System.exit(0);		// Siehe Kommentar unten zu workbench.close();
		}

		// u.a. Beenden des Splash Screens, damit dieser nicht einen möglichen Lizenzdialog überdeckt
		appContext.applicationRunning();

		// Lizenz lesen
		boolean	success	= licenseHandler.readLicense();

		if (commandLine.silentSetupRequest()) {
			// ProxyHandler und LicenseManager sollten ihre jeweiligen Parameter bereits ausgewertet haben!
			proxyHandler.persist();
			System.exit(0);		// Siehe Kommentar unten zu workbench.close();
		}

		// Unter Umständen wiederholte Lizenzprüfung
		while (!success || success && !licenseHandler.checkLicense()) {

			// Das Lizenzobjekt wird automatisch neu injected, wenn eine Änderung passiert
			if (license != null && license.isMaintenanceExpired()) {
				// Fehlermeldung und Programmende
				MessageDialog.openError(null, "Maintenance expired", "License key is for former releases of Lookup2 up to " + DateFormat.getDateInstance(DateFormat.MEDIUM).format(license.getLicenseMaintenanceExpireDate()) + " only."
																					+ System.lineSeparator() + System.lineSeparator()
																					+ "Please re-activate your license in the following dialog to check if newer license information are available. You may also need to re-apply any modification key you've got on your own.");
			}

			// Lizenzdialog öffnen, und zwar nur mit der Lizenzseite
			if (showLicenseDialogOnly(null, context) == Window.OK) {
				success	= true;

			} else {
				MessageDialog.openInformation(null, Messages.getString("LICENSE_CHECK_MESSAGE_TITLE"), Messages.getString("LICENSE_CHECK_NOT_SUCCESSFUL"));

				// Der Plan war, die IWorkbench an dieser Stelle geregelt herunterzufahren, anstatt sie hart abzuschießen.
				// Leider wird das bis heute nicht unterstützt, in dem Sinn, dass im e4-PostContextCreate schlicht und
				// ergreifend (noch) kein IWorkbench-Objekt verfügbar ist. Und ohne dieses ist keine andere Methode ver-
				// fügbar, um die Anwendung geregelt zu schließen.
				// Es gibt anscheinend eine solche Lösung für E(fx)clipse, aber leider nicht für SWT:
				// https://stackoverflow.com/questions/28507982/eclipse-rcp-how-to-shutdown-before-workbench-initializes
				// https://tomsondev.bestsolution.at/2014/11/03/efxclipse-1-1-new-features-api-to-restart-your-e4-app-on-startup/
//				workbench.close();
				System.exit(0);
			}
		}
	}

	@PreSave
	void shutDown(LicenseHandler licenseHandler) {
	}

	static int showLicenseDialogOnly(Shell shell, IEclipseContext context) {
		PreferenceManager	manager	= new PreferenceManager();
		MyPrefPage	licPrefPage	= new LicensePreferencePage();
		ContextInjectionFactory.inject(licPrefPage, context);
		manager.addToRoot(new PreferenceNode(licPrefPage.getTitle(), licPrefPage));

		PreferenceDialog 	prefDialog	= new PreferenceDialog(shell, manager);
		prefDialog.setSelectedNode(licPrefPage.getTitle());

		MyPrefPage	proxyPrefPage	= new ProxyPreferencePage();
		ContextInjectionFactory.inject(proxyPrefPage, context);
		manager.addToRoot(new PreferenceNode(proxyPrefPage.getTitle(), proxyPrefPage));

		prefDialog.create();
		prefDialog.getTreeViewer().setComparator(new ViewerComparator());
		prefDialog.getTreeViewer().expandAll();

		return prefDialog.open();
	}

	// Wiederherstellen der Fensterposition (nur bei im Vergleich zum letzten Mal gleichen Bildschirmtelemetriedaten)
	static void restoreMainWindowPosition(EModelService modelService, MWindow mainWindow) {
		// Unterschiedliche Anzahl Monitore? Abbruch
		if (lifeCyclePreferences.getInt(DISPLAY_MONITORS_NODE, -1) != Display.getCurrent().getMonitors().length)
			return;

		// Unterschiedliche Startposition des Hauptmonitors (z.B. wegen anderer Monitorreihenfolge)? Abbruch
		if (lifeCyclePreferences.getInt(DISPLAY_BOUNDS_X_NODE, -1) != Display.getCurrent().getBounds().x
				|| lifeCyclePreferences.getInt(DISPLAY_BOUNDS_Y_NODE, -1) != Display.getCurrent().getBounds().y
				|| lifeCyclePreferences.getInt(DISPLAY_BOUNDS_W_NODE, -1) != Display.getCurrent().getBounds().width
				|| lifeCyclePreferences.getInt(DISPLAY_BOUNDS_H_NODE, -1) != Display.getCurrent().getBounds().height)
			return;

		// Unterschiedliche DPI-Anzahl (anderer Monitor)? Abbruch
		if (lifeCyclePreferences.getInt(DISPLAY_DPI_NODE, -1) != Display.getCurrent().getDPI().x)
			return;

		// Alles ok, Wiederherstellen der letzten Fensterposition des Hauptfensters
		mainWindow.setX(     lifeCyclePreferences.getInt(WINDOW_XPOS_NODE   + "_" + mainWindow.getElementId(), 0));
		mainWindow.setY(     lifeCyclePreferences.getInt(WINDOW_YPOS_NODE   + "_" + mainWindow.getElementId(), 0));
		mainWindow.setWidth( lifeCyclePreferences.getInt(WINDOW_WIDTH_NODE  + "_" + mainWindow.getElementId(), 1280));
		mainWindow.setHeight(lifeCyclePreferences.getInt(WINDOW_HEIGHT_NODE + "_" + mainWindow.getElementId(), 800));
	}

	// Abspeichern der letzten Fensterposition und der wichtigsten Bildschirmtelemetriedaten
	static void persistWindowPosition(MWindow mainWindow) {
		try {
			lifeCyclePreferences.putInt(WINDOW_XPOS_NODE + "_" + mainWindow.getElementId(), mainWindow.getX());
			lifeCyclePreferences.putInt(WINDOW_YPOS_NODE + "_" + mainWindow.getElementId(), mainWindow.getY());
			lifeCyclePreferences.putInt(WINDOW_WIDTH_NODE + "_" + mainWindow.getElementId(), mainWindow.getWidth());
			lifeCyclePreferences.putInt(WINDOW_HEIGHT_NODE + "_" + mainWindow.getElementId(), mainWindow.getHeight());

			lifeCyclePreferences.putInt(DISPLAY_MONITORS_NODE, Display.getCurrent().getMonitors().length);
			lifeCyclePreferences.putInt(DISPLAY_BOUNDS_X_NODE, Display.getCurrent().getBounds().x);
			lifeCyclePreferences.putInt(DISPLAY_BOUNDS_Y_NODE, Display.getCurrent().getBounds().y);
			lifeCyclePreferences.putInt(DISPLAY_BOUNDS_W_NODE, Display.getCurrent().getBounds().width);
			lifeCyclePreferences.putInt(DISPLAY_BOUNDS_H_NODE, Display.getCurrent().getBounds().height);
			lifeCyclePreferences.putInt(DISPLAY_DPI_NODE, Display.getCurrent().getDPI().x);

			lifeCyclePreferences.flush();
		} catch (BackingStoreException e) {
		}
	}

}
