package de.credo.lookup2;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.List;

import de.credo.common.bundles.PathUtils;

public class StartupFileHandler {

	private	static final	String	fileName	= "FilesToOpenWithLookup2.tmp";

	public static Path getStartupFile() {
		return PathUtils.getUserConfigDir().resolve(fileName);
	}

	public static void createAndWrite(List<Path> files) {
		if (files == null || files.isEmpty())
			return;

		try (BufferedWriter bw = Files.newBufferedWriter(getStartupFile(), StandardCharsets.UTF_8, StandardOpenOption.WRITE, StandardOpenOption.CREATE, StandardOpenOption.APPEND)) {
			for (Path file: files) {
				bw.write(file.toAbsolutePath().toString());
				bw.newLine();
			}
		} catch (IOException e) {
			// Tue nichts. Dann kann die zu öffnende Datei halt nicht ins Startup-File geschrieben
			// werden. Eine Fehlermeldung hilft hier auch nicht.
		}
	}

	public static List<Path> readAndDelete() {
		if (!Files.exists(getStartupFile()))
			return null;

		List<Path>	files	= new ArrayList<>();

		try (BufferedReader br = Files.newBufferedReader(getStartupFile(), StandardCharsets.UTF_8)) {
			br.lines().forEach(f -> {
				Path	file	= Paths.get(f);
				if (Files.exists(file)) {
					files.add(file);
				}
			});
		} catch (IOException e) {
			// Probleme beim Lesen? Dann halt nichts tun. Die Datei wird im folgenden Step so oder so gelöscht
		}

		try {
			Files.deleteIfExists(getStartupFile());
		} catch (IOException e) {
		}

		return files;
	}
}
