package de.credo.lookup2;

import java.util.stream.Stream;

import org.eclipse.e4.ui.model.application.MApplicationElement;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.model.application.ui.menu.MMenu;
import org.eclipse.swt.widgets.Shell;
import org.osgi.framework.FrameworkUtil;

import de.credo.lookup2.viewers.api.L2Viewer;
import de.credo.lookup2.viewers.api.L2ViewerFactory;

public class ToolbarUtils {

	public static void hideViewerToolbars(MTrimmedWindow window) {
		((Shell) window.getWidget()).setRedraw(false);
		showViewerToolbars(window, "ANonNullDummyViewerBundleName", false);
		((Shell) window.getWidget()).setRedraw(true);
	}

	public static void showViewerToolbars(MTrimmedWindow window, L2Viewer viewer) {
		((Shell) window.getWidget()).setRedraw(false);
		showViewerToolbars(window, viewer, true);
		((Shell) window.getWidget()).setRedraw(true);
	}

	private static void showViewerToolbars(MTrimmedWindow window, L2Viewer viewer, boolean show) {
		if (viewer != null) {
			String	bundleName	= FrameworkUtil.getBundle(viewer.getClass()).getSymbolicName();
			showViewerToolbars(window, bundleName, show);
		} else {
			showViewerToolbars(window, "ANonNullDummyViewerBundleName", false);
		}
	}

	private static void showViewerToolbars(MTrimmedWindow window, String bundleName, boolean show) {
//		String wStr	= window.toString();
//		System.out.println("showViewerToolbars\t" + System.currentTimeMillis() + "\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + bundleName + "\t" + show);

		String	thisBundleName	= FrameworkUtil.getBundle(ToolbarUtils.class).getSymbolicName();

		window.getTrimBars().stream()
				.filter(tb -> tb.getElementId().equals(Constants.VIEWERS_TRIMBAR))			// Reduziert auf die eine Viewer-TrimBar
				.flatMap(tb -> tb.getChildren().stream())									// Liefert dessen ToolBars und ToolControls (alle)
				.filter(te -> ((MApplicationElement) te).getContributorURI() != null		// Generell nur Trimbars in anderen Plugins beglücken
				          && !((MApplicationElement) te).getContributorURI().endsWith(thisBundleName))
				.flatMap(te -> Stream.of(te))												// Stream-in-Stream: Holen der TrimElements
		// Scheint inzwischen unnötig zu sein. Aus Dokumentationszwecken noch drin
		//		.flatMap(te -> Stream.concat(Stream.of(te), Stream.of(te)					// Stream-in-Stream: Sowohl die TrimElements
		//			.filter(tbe -> tbe instanceof MElementContainer)						// ... als auch, im Fall einer Toolbar deren ...
		//			.flatMap(tbe -> ((MToolBar) tbe).getChildren().stream())))				// ... Children, sind zu berücksichtigen
				.filter(te -> te.isToBeRendered())											// Nur aktive Objekte
				.forEach(te -> {
//					String wStr	= window.toString();
//					System.out.println("showViewerToolbars\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + bundleName + "\t" + show + "\t" + te.getContributorURI() + "\t" + te.getElementId() + "\t" + (show ? te.getContributorURI().endsWith(bundleName) : false));
					te.setVisible(show ? te.getContributorURI().endsWith(bundleName) : false);
				});
	}

	public static void hidePerspectiveToolbars(MTrimmedWindow window) {
		((Shell) window.getWidget()).setRedraw(false);
		showPerspectiveToolbars(window, "ANonNullDummyPerspectiveId", false);
		((Shell) window.getWidget()).setRedraw(true);
	}

	public static void showPerspectiveToolbars(MTrimmedWindow window, String perspectiveId, boolean show) {
//		String wStr	= window.toString();
//		System.out.println("showPerspectiveToolbars\t" + System.currentTimeMillis() + "\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + perspectiveId + "\t" + show);

		window.getTrimBars().stream()
				.filter(tb -> tb.getElementId().equals(Constants.MAIN_TRIMBAR))				// Reduziert auf die eine Haupt-TrimBar
				.flatMap(tb -> tb.getChildren().stream())									// Liefert dessen ToolBars und ToolControls (alle)
				.flatMap(te -> Stream.of(te))												// Stream-in-Stream: Holen der TrimElements
				.filter(te -> te.getTags().contains(Constants.RELATED_TO_PERSPECTIVE_FLAG))	// Nur wirklich eine Perspektive betreffende Elemente
				.filter(te -> te.isToBeRendered())											// Nur aktive Objekte
				.forEach(te -> {
//					String wStr	= window.toString();
//					System.out.println("showPerspectiveToolbars\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + perspectiveId + "\t" + show + "\t" + te.getContributorURI() + "\t" + te.getElementId() + "\t" + (show ? te.getPersistedState().get(Constants.PERSPECTIVE_ID_KEY).equals(perspectiveId) : false));
					te.setVisible(show ? te.getPersistedState().get(Constants.PERSPECTIVE_ID_KEY).equals(perspectiveId) : false);
				});
	}

	// TODO Nicht ausreichend! Eigentlich muss es eigene Routinen für Viewer- und Parser-Menüs geben
	public static void hideAllMenus(MTrimmedWindow window) {
		showMenus(window, "ANonNullDummyViewerBundleName", false);
	}

	public static void showViewerMenus(MTrimmedWindow window, L2Viewer viewer) {
		showViewerMenus(window, viewer, true);
	}

	private static void showViewerMenus(MTrimmedWindow window, L2Viewer viewer, boolean show) {
		if (viewer != null) {
			String	bundleName	= FrameworkUtil.getBundle(viewer.getClass()).getSymbolicName();
			showMenus(window, bundleName, show);
		} else {
			showMenus(window, "ANonNullDummyViewerBundleName", false);
		}
	}

	public static void showParserMenus(MTrimmedWindow window, L2ViewerFactory factory) {
		showParserMenus(window, factory, true);
	}

	private static void showParserMenus(MTrimmedWindow window, L2ViewerFactory factory, boolean show) {
		if (factory != null) {
			String	bundleName	= FrameworkUtil.getBundle(factory.getClass()).getSymbolicName();
			showMenus(window, bundleName, show);
		} else {
			showMenus(window, "ANonNullDummyViewerBundleName", false);
		}
	}

	private static void showMenus(MTrimmedWindow window, String bundleName, boolean show) {
//		String wStr	= window.toString();
//		System.out.println("showMenus\t\t" + System.currentTimeMillis() + "\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + bundleName + "\t" + show);

		String	thisBundleName	= FrameworkUtil.getBundle(ToolbarUtils.class).getSymbolicName();

		window.getMainMenu().getChildren().stream()
			.flatMap(mme -> Stream.concat(Stream.of(mme), Stream.of(mme)				// Stream-in-Stream: Sowohl die MainMenuElements
				.filter(sme -> sme instanceof MElementContainer)						// ... als auch deren Kinder ...
				.flatMap(sme -> ((MMenu) sme).getChildren().stream())))					// ... sind zu berücksichtigen
			.filter(te -> ((MApplicationElement) te).getContributorURI() != null		// Generell nur Fremd-Menüs berücksichtigen, die eigenen ausschließen
			          && !((MApplicationElement) te).getContributorURI().endsWith(thisBundleName))
			.forEach(me -> {
//				String wStr	= window.toString();
//				System.out.println("showMenus\t\t" + wStr.substring(wStr.indexOf('@') - 1, wStr.indexOf(' ')) + "\t" + bundleName + "\t" + show + "\t" +  me.getElementId() + "\t" + (show ? me.getContributorURI().endsWith(bundleName) : false));
				me.setVisible(show ? me.getContributorURI().endsWith(bundleName) : false);
			});
	}
}
