package de.credo.lookup2;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.model.application.ui.basic.MWindowElement;
import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.IWindowCloseHandler;

import de.credo.lookup2.parsers.impl.RootElement;

class WindowCloseHandler implements IWindowCloseHandler {

	@Override
	public boolean close(MWindow window) {
		if (!(window instanceof MTrimmedWindow))
			return true;

		IEclipseContext		context				= window.getContext();
		MApplication		application			= context.get(MApplication.class);
		IWorkbench			workbench			= context.get(IWorkbench.class);
		EPartService		partService			= context.get(EPartService.class);
		RootElement			rootElement			= context.get(RootElement.class);
		GlobalAppManager	globalAppManager	= application.getContext().get(GlobalAppManager.class);

		// Läuft der Parser-Job noch (wenn vorhanden)? Dann wird der beim ersten Schließversuch einfach beendet
		if (rootElement != null) {
			Job	readJob	= rootElement.getReadJob();
			if (readJob != null && readJob.getState() != Job.NONE) {
				readJob.cancel();
				return false;
			}
		}

		// Speichern des aktuellen Unterfensters (wenn Dirty und mit vorheriger Rückfrage)
		for (MWindowElement mwe: window.getChildren()) {
			if (mwe instanceof MPart && ((MPart) mwe).isDirty()) {
				if (!partService.savePart((MPart) mwe, true))
					return false;
			}
		}

		// Schließen des Streams. Der PreDestroy-Handler ist hier unzuverlässig, weil es theoretisch sein kann, dass
		// weder die Objects- noch die Structure-Perspektive jemals wirklich geladen/initialisiert wurde (in MultiDoc
		// zum Beispiel, oder wenn nur die Viewer-Perspektive aktiv war.)
		if (rootElement != null) {
			rootElement.closeStream();
		}

		// Schließen der kompletten Anwendung, wenn das letzte sichtbare Fenster geschlossen wird. Abfrage auf "2",
		// weil es immer ein unsichtbares Dummy-Fenster gibt, das wegen der restlichen Steuerung per Snippets nötig ist.
		if (application.getChildren().size() == 2) {

			// Ist noch eine Datei im letzten Fenster geöffnet? Dann wird beim zweiten Schließversuch zunächst mal nur
			// diese geschlossen, bevor das Fenster endgültig geschlossen wird. Je nach Voreinstellung.
			if (rootElement != null) {
				globalAppManager.closeFile((MTrimmedWindow) window, true);
				if (globalAppManager.getTwoStepCloseMode())
					return false;
			}

			// Fensterpositionen des Hauptfensters für den LifeCycle speichern
			LifeCycle.persistWindowPosition(window);

			// Abräumen des ModuleHandlers, der wiederum die Factories abräumt. Das ist eigentlich nur wichtig für die
			// MultiViewFactory/MultiViewEngine
			globalAppManager.dispose();

			return workbench.close();
		}

		return true;
	}

}
