package de.credo.lookup2;

import java.util.List;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBar;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.workbench.IPresentationEngine;
import org.eclipse.e4.ui.workbench.UIEvents;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.IWindowCloseHandler;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.credo.common.bundles.messages.CdoLogSink;
import de.credo.lookup2.plugins.api.L2ProgressMonitor;

public class WindowCreationObserver implements EventHandler {

	private	EModelService	modelService;

	WindowCreationObserver(EModelService modelService) {
		this.modelService	= modelService;
	}

	@Override
	public void handleEvent(Event event) {
		if (!UIEvents.isSET(event))
			return;

		if (event.getProperty(UIEvents.EventTags.ELEMENT) instanceof MTrimmedWindow) {
			MTrimmedWindow	window		= (MTrimmedWindow) event.getProperty(UIEvents.EventTags.ELEMENT);
			IEclipseContext	wContext	= window.getContext();
//			EPartService	partService	= context.get(EPartService.class);

			// WindowClose-Handler in den Context des/jeden neuen Fensters setzen
			wContext.set(IWindowCloseHandler.class, new WindowCloseHandler());

			// LogMessage-Handler in den Context des/jeden neuen Fensters setzen
			wContext.set(CdoLogSink.class, ContextInjectionFactory.make(CdoLogSink.class, wContext));

			// LogMessage-Handler in den Context des/jeden neuen Fensters setzen
			wContext.set(L2ProgressMonitor.class, ContextInjectionFactory.make(GlobalProgressMonitor.class, wContext));

			// Save-Handler in den Context setzen
			// wContext.set(ISaveHandler.class, new ISaveHandler() {
			// });

			// Übernehmen der alternativen Icon-Sets für Disabled, Selected, Unselected-Icons
			manipulateIcons(modelService, window);
		}
	}

	// Übernehmen des disabledIconURI aus den Persisted Data in die TransientDatamap, wo sie hingehören.
	// Siehe: https://www.eclipse.org/forums/index.php/m/1771855/#msg_1771855
	private void manipulateIcons(EModelService modelService, MTrimmedWindow window) {
		// ... für die TrimBars bzw. die darin liegenden ToolBars
		List<MToolBar> toolBars = modelService.findElements(window, null, MToolBar.class, null);
		for (MToolBar toolBar : toolBars) {
			manipulateToolBarIcons(toolBar);
		}

		// Jetzt auch noch die Icons in den ToolBars der Parts anfassen. Die werden interessanterweise oben nicht gefunden
		List<MPart> parts = modelService.findElements(window, null, MPart.class, null);
		for (MPart part : parts) {
			manipulateToolBarIcons(part.getToolbar());
		}
	}

	private void manipulateToolBarIcons(MToolBar toolBar) {
		if (toolBar != null) {
			for (MToolBarElement toolBarElement : toolBar.getChildren()) {
				if (toolBarElement.getPersistedState().containsKey(Constants.DISABLED_ICON_URI_KEY)) {
					toolBarElement.getTransientData().put(IPresentationEngine.DISABLED_ICON_IMAGE_KEY,
							toolBarElement.getPersistedState().get(Constants.DISABLED_ICON_URI_KEY));
				}
			}
		}
	}
}