package de.credo.lookup2.controls;

import java.nio.charset.Charset;

import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.credo.common.bundles.messages.CdoLogger;
import de.credo.lookup2.Constants;
import de.credo.lookup2.Messages;
import de.credo.lookup2.parsers.api.SingleByteCharsets;

public class EncodingSelectionControl {

	private	final static	CdoLogger	LOGGER	= new CdoLogger(Constants.LOGGER_ORIGIN_HEXVIEW);

	@Inject	IEclipseContext	context;
	@Inject	@Translation	Messages	messages;

	private	Combo	combo;

	@PostConstruct
	public void postConstruct(Composite parent, Display display, IEventBroker eventBroker, @Active MTrimmedWindow window, @Optional Charset newEncoding) {
		Composite	composite	= new Composite(parent, SWT.NONE);
		GridLayout	gl	= new GridLayout();
		gl.marginHeight	= 0;
		gl.marginWidth	= 0;
		composite.setLayout(gl);

		combo	= new Combo(composite, SWT.RIGHT | SWT.READ_ONLY);

//		Font	smallerFont	= JFaceResources.getDefaultFontDescriptor().increaseHeight(-1).createFont(display);
		Font	smallerFont	= FontDescriptor.createFrom(JFaceResources.getDefaultFont()).increaseHeight(-1).createFont(display);

		combo.setFont(smallerFont);
		combo.setVisibleItemCount(15);

		String[]	friendlyNames	= new String[SingleByteCharsets.values().length];
		for (int i = 0; i < SingleByteCharsets.values().length; i++) {
			friendlyNames[i]	= SingleByteCharsets.values()[i].friendlyName();
		}
		combo.setItems(friendlyNames);

		combo.addDisposeListener(e -> smallerFont.dispose());

		combo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				if (combo.getSelectionIndex() >= 0) {
					Charset	encoding	= Charset.forName(SingleByteCharsets.values()[combo.getSelectionIndex()].getCsName());
					if (encoding != null) {
						context.set(Charset.class, encoding);
					} else {
						LOGGER.warn(messages.encodingSelectionControlEncodingNotFound);
					}
				}
			}
		});

		if (newEncoding != null) {
			setEncoding(newEncoding);
		}
	}

	@Inject
	public void setEncoding(@Optional Charset newEncoding) {
		if (combo == null)
			return;

		if (newEncoding != null) {
			SingleByteCharsets	sbcs	= SingleByteCharsets.from(newEncoding);
			if (sbcs == null) {
				sbcs	= SingleByteCharsets.Windows1252;
			}
			combo.setText(sbcs.friendlyName());
		}
	}
}
