package de.credo.lookup2.handlers;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.workbench.IWorkbench;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

public class CloseAppHandler<IHandlerService> {

	@Execute
	public void execute(IWorkbench workbench, Shell shell, EPartService partService, @Active MTrimmedWindow window) {
		// Macht am Ende nur das Fenster unsichtbar, es bleibt aber im Application Model erhalten.
		// Das ist die in Foren zu findende Lösung, um ein Fenster zu schließen. Leider löst sie
		// nicht den IWindowCloseHandler aus.
//		window.setToBeRendered(false);

		// Dieser Befehl ist da viel stringenter: Er ist nämlich am Ende der gleiche, der auch
		// ausgelöst wird, wenn das X am Fenster gedrückt wird. Damit lässt sich die Verarbeitung
		// schön zentralisieren.
		shell.close();

		// Zum Nachlesen:
		// https://github.com/eclipse/eclipse.platform.ui/blob/master/bundles/org.eclipse.e4.ui.workbench.renderers.swt/src/org/eclipse/e4/ui/workbench/renderers/swt/WBWRenderer.java
		// in der Routine hookControllerLogic() wird der ShellListener gesetzt
		// Default-Implementierung des IWindowCloseHandlers ist in setCloseHandler()
		// Wir nutzen unseren eigenen IWindowCloseHandler
	}

	// @CanExecute
	// public boolean canExecute() {
	// return true;
	// }
}
