package de.credo.lookup2.handlers;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.workbench.UIEvents;
import org.eclipse.e4.ui.workbench.modeling.EModelService;

import de.credo.common.trees.ITreeParentNode;
import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.History;
import de.credo.lookup2.parsers.api.objects.L2DOMObject;
import de.credo.lookup2.parsers.api.structure.L2ParentElement;
import de.credo.lookup2.plugins.impl.PerspectiveTypes;

public class HistoryForthHandler {

	@Execute
	public void execute(History history, GlobalAppManager globalAppManager, @Active MTrimmedWindow window, IEventBroker eventBroker, EModelService modelService) {
		ITreeParentNode<?, ?>	currentObject	= history.replayForth();

		if (currentObject instanceof L2ParentElement) {
			window.getContext().set(L2ParentElement.class, (L2ParentElement) currentObject);
			if (PerspectiveTypes.from(modelService.getActivePerspective(window).getElementId()) != PerspectiveTypes.STRUCTURE) {
				globalAppManager.switchPerspective(window, PerspectiveTypes.STRUCTURE);
			}

		} else if (currentObject instanceof L2DOMObject) {
			window.getContext().set(L2DOMObject.class, (L2DOMObject) currentObject);
			if (PerspectiveTypes.from(modelService.getActivePerspective(window).getElementId()) != PerspectiveTypes.OBJECTS) {
				globalAppManager.switchPerspective(window, PerspectiveTypes.OBJECTS);
			}
		}

		// Händisches Auslösen der CanExecute-Abfrage in den Handlern. Diese wird seit Kepler nicht mehr als Timer ausgeführt, sondern
		// nur noch bei relevanten Änderungen an der Oberfläche, wie Partwechseln usw. Nötig zum Update des SearchAgain-Buttons
		eventBroker.send(UIEvents.REQUEST_ENABLEMENT_UPDATE_TOPIC, UIEvents.ALL_ELEMENT_ID);
	}

	@CanExecute
	public boolean canExecute(@Optional History history) {
		if (history != null)
			return history.hasFutureElements();
		else
			return false;
	}

}
