package de.credo.lookup2.handlers;

import java.nio.file.Path;
import java.nio.file.Paths;

import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.osgi.service.prefs.BackingStoreException;

import de.credo.common.bundles.PathUtils;
import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.plugins.api.L2FileData;
import de.credo.lookup2.plugins.impl.FileData;

public class OpenFileHandler {

	private	final static	String				MRU_FOLDER		= "Most recently used file folder";

	private	final static	IEclipsePreferences	userPreferences	= InstanceScope.INSTANCE.getNode(OpenFileHandler.class.getName());

	@Execute
	public void execute(Shell shell, GlobalAppManager globalAppManager, IEventBroker eventBroker, @Optional L2FileData openFile, @Active MTrimmedWindow window) {
		FileDialog	fileDialog	= new FileDialog(shell, SWT.MULTI);
		fileDialog.setFilterPath(userPreferences.get(MRU_FOLDER, PathUtils.getUserDir().toAbsolutePath().toString()));
		String		fileToOpen	= fileDialog.open();

		if (fileToOpen != null && fileDialog.getFileNames().length > 0) {
			Path	basePath	= Paths.get(fileToOpen).getParent();

			try {
				userPreferences.put(MRU_FOLDER, basePath.toAbsolutePath().toString());
				userPreferences.flush();
			} catch (BackingStoreException e) {
			}

			// Sehr ähnliche Routine in AppStartupCompleteObserver!!
			for (int i = 0; i < fileDialog.getFileNames().length; i++) {
				L2FileData	fileData	= new FileData(basePath.resolve(fileDialog.getFileNames()[i]), false);
				if (i == 0 && (openFile == null || openFile.getFile() == null)) {
					// Erste Datei im aktuellen Fenster ...
					globalAppManager.openInWindow(window, fileData, SwitchPerspectiveTargetTypes.ANY);
				} else {
					// ... und weitere Dateien in weiteren, zusätzlichen Fenstern öffnen
					globalAppManager.openInNewWindow(fileData, true, SwitchPerspectiveTargetTypes.ANY);
				}
			}
		}
	}

	@CanExecute
	public boolean canExecute(@Optional L2FileData fileData, GlobalAppManager globalAppManager) {
		// Leichte Vergewaltigung dieses canExecute-Handlers, weil ein Objekt gebraucht wird, dem ein aktuelles
		// FileData-Objekt injected werden kann und das in jedem Fenster sichtbar ist. Welches UI-Element ist schon
		// immer sichtbar, wenn nicht dieser Button hier?
		globalAppManager.storeFileHistory(fileData);

		// ansonsten normal weiter
		return true;
	}
}
