
package de.credo.lookup2.handlers.files;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import jakarta.inject.Inject;
import jakarta.inject.Named;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.widgets.Shell;

import de.credo.common.bundles.messages.CdoLogger;
import de.credo.lookup2.Messages;
import de.credo.lookup2.filetree.api.FileTreeConstants;
import de.credo.lookup2.filetree.api.FileTreeNodeFile;
import de.credo.lookup2.filetree.api.FileTreeNodeFileSystem;
import de.credo.lookup2.parts.files.FileTreePart;

public class DeleteFileHandler {

	private	final static	CdoLogger	LOGGER	= new CdoLogger(FileTreeConstants.LOGGER_ORIGIN_FILESYSTEM);

	@Inject	@Translation	Messages	messages;

	@Execute
	public void execute(@Active MPart filePart, @Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection, Shell shell, @Active MTrimmedWindow window) {
		List<FileTreeNodeFile>	files	= new ArrayList<>();

		for (Object s: selection.toArray()) {
			if (s instanceof FileTreeNodeFile && !(s instanceof FileTreeNodeFileSystem)) {
				FileTreeNodeFile	thisFile	= (FileTreeNodeFile) s;
				files.add(thisFile);
			}
		}

		if (!files.isEmpty()) {
			String	msg = files.size() > 1 ? messages.fileTreeContextMenuDeleteFileDialog2 : messages.fileTreeContextMenuDeleteFileDialog1;
			msg	= msg.replace("%1", files.stream()
					.map(f -> f.getName())
					.collect(Collectors.joining(", ", "'", "'")));

			if (MessageDialog.openQuestion(shell, messages.fileTreeContextMenuDeleteFileTitle, msg)) {
				for (FileTreeNodeFile f: files) {
					try {
						if (f.isFile()) {
							((FileTreePart) filePart.getObject()).closeFile(f.getPath());
						}
						f.delete();
					} catch (Exception e) {
						LOGGER.error(window.hashCode(), e.getLocalizedMessage(), e);
					}
				}
			}
		}

	}

	@CanExecute
	public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection) {
		for (Object s: selection.toArray()) {
			if (s instanceof FileTreeNodeFile && !(s instanceof FileTreeNodeFileSystem) && (((FileTreeNodeFile) s).isFile() || ((FileTreeNodeFile) s).isFolder()))
				return true;
		}

		return false;
	}

}