
package de.credo.lookup2.handlers.files;

import java.io.IOException;

import jakarta.inject.Named;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.model.application.ui.menu.MDirectMenuItem;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.viewers.ITreeSelection;

import de.credo.lookup2.Constants;
import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.filetree.api.FileTree;
import de.credo.lookup2.filetree.api.FileTreeConstants;
import de.credo.lookup2.filetree.api.FileTreeNodeFile;
import de.credo.lookup2.plugins.impl.FileData;
import de.credo.lookup2.plugins.impl.ModulesHandler;

public class OpenAsTypeHandler {

	@Execute
	public void execute(MDirectMenuItem menuItem, @Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection, IEventBroker eventBroker, @Active MTrimmedWindow window, GlobalAppManager globalAppManager, ModulesHandler modulesHandler, FileTree fileTree) {
		for (Object s: selection.toArray()) {
			if (s instanceof FileTreeNodeFile) {
				// Ob dieser Name gespeichert wird, oder nicht, entscheidet sich beim Empfänger
				eventBroker.post(FileTreeConstants.ADD_TO_QUICK_SELECT_LIST, ((FileTreeNodeFile) s).getParent().getFullName());

				try {
					// Zusätzliches Setzen des MediaTypes, damit Übergehen der automatischen Dateityperkennung
					FileData	fileData	= new FileData(((FileTreeNodeFile) s).getPath(), ((FileTreeNodeFile) s).isWritable(), modulesHandler.getFormat(menuItem.getLabel()));
					if (menuItem.getTags().contains(Constants.OPEN_AS_IN_NEW_WINDOW_TAG)) {
						globalAppManager.openInNewWindow(fileData, false, SwitchPerspectiveTargetTypes.ANY);
					} else {
						globalAppManager.openInWindow(window, fileData, fileTree.isAutoSwitchPerspective() ? SwitchPerspectiveTargetTypes.ANY : SwitchPerspectiveTargetTypes.NONE);
					}
				} catch (IOException e) {
				}
			}
		}
	}

	@CanExecute
	public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection) {
		for (Object s: selection.toArray()) {
			if (s instanceof FileTreeNodeFile && ((FileTreeNodeFile) s).isFile())
				return true;
		}

		return false;
	}

}