
package de.credo.lookup2.handlers.files;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import jakarta.inject.Inject;
import jakarta.inject.Named;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import de.credo.common.bundles.messages.CdoLogger;
import de.credo.lookup2.Messages;
import de.credo.lookup2.filetree.api.FileTree;
import de.credo.lookup2.filetree.api.FileTreeConstants;
import de.credo.lookup2.filetree.api.FileTreeNode;
import de.credo.lookup2.filetree.api.FileTreeNodeFile;
import de.credo.lookup2.parts.files.FileTreePart;

public class PasteFileHandler {

	private	final static	CdoLogger	LOGGER	= new CdoLogger(FileTreeConstants.LOGGER_ORIGIN_FILESYSTEM);

	@Inject	@Translation	Messages	messages;

	@Execute
	public void execute(@Active MPart filePart, @Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection, Display display, FileTree fileTree, @Active MTrimmedWindow window, UISynchronize synchronizer, Shell shell) {
		Clipboard	clipboard	= new Clipboard(display);
		String		text		= (String) clipboard.getContents(TextTransfer.getInstance());

		if (text != null) {
			List<FileTreeNodeFile>	files	= new ArrayList<>();

			for (String f: text.split(System.lineSeparator())) {
				try {
					FileTreeNode	node	= fileTree.getNode(f);
					if (node instanceof FileTreeNodeFile && node.exists()) {
						files.add((FileTreeNodeFile) node);
					}
				} catch(IOException e) {
				}
			}

			if (!files.isEmpty()
					&& !selection.isEmpty()
					&& selection.getFirstElement() instanceof FileTreeNodeFile
					&& ((FileTreeNodeFile) selection.getFirstElement()).isFolder()
					&& ((FileTreeNodeFile) selection.getFirstElement()).isWritable()) {
				final	FileTreeNodeFile	target	= (FileTreeNodeFile) selection.getFirstElement();

				// Sieht komisch aus, funktioniert aber, weil das erste target-Objekt intern gar nicht überprüft wird
				if (target.canBeCopiedTo(target)) {
					for (FileTreeNodeFile f: files) {
						boolean	copied	= false;
						String	newName	= f.getName();

						while (!copied) {
							try {
								FileTreeNodeFile	node	= target.resolveChild(newName);
								if (node.exists()) {
									InputDialog	inputDialog	= new InputDialog(shell,
											messages.fileTreeContextMenuPasteTitle,
											messages.fileTreeContextMenuPasteDialog.replace("%1", newName),
											newName,
											newText -> newText.isEmpty() ? messages.fileTreeContextMenuPasteError : null);
									inputDialog.setBlockOnOpen(true);

									if (inputDialog.open() == Window.OK) {
										newName	= inputDialog.getValue();
									} else {
										break;
									}
								} else {
									f.copyTo(node);
									copied	= true;
								}
							} catch (Exception e) {
								LOGGER.error(window.hashCode(), e.getLocalizedMessage(), e);
							}
						}
					}

					target.reload();
					((FileTreePart) filePart.getObject()).refresh(target);
				}
			}
		}
	}

	@CanExecute
	public boolean canExecute(Display display, @Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection, FileTree fileTree) {
		if (!selection.isEmpty()
				&& selection.getFirstElement() instanceof FileTreeNodeFile
				&& ((FileTreeNodeFile) selection.getFirstElement()).isFolder()
				&& ((FileTreeNodeFile) selection.getFirstElement()).isWritable()) {

			Clipboard	clipboard	= new Clipboard(display);
			String		text		= (String) clipboard.getContents(TextTransfer.getInstance());

			if (text != null) {
				List<FileTreeNodeFile>	files	= new ArrayList<>();

				for (String f: text.split(System.lineSeparator())) {
					try {
						FileTreeNode	node	= fileTree.getNode(f);
						if (node instanceof FileTreeNodeFile && node.exists()) {
							files.add((FileTreeNodeFile) node);
						}
					} catch(IOException e) {
					}
				}

				if (!files.isEmpty())
					return true;
			}
		}

		return false;
	}

}