
package de.credo.lookup2.handlers.files;

import jakarta.inject.Named;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;

import de.credo.common.bundles.messages.CdoLogger;
import de.credo.lookup2.Messages;
import de.credo.lookup2.filetree.api.FileTreeConstants;
import de.credo.lookup2.filetree.api.FileTreeNode;
import de.credo.lookup2.filetree.api.FileTreeNodeFile;
import de.credo.lookup2.parts.files.FileTreePart;

public class RenameFileHandler {

	private	final static	CdoLogger	LOGGER	= new CdoLogger(FileTreeConstants.LOGGER_ORIGIN_FILESYSTEM);

	@Execute
	public void execute(@Active MPart filePart, @Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection, Shell shell, @Translation Messages messages, @Active MTrimmedWindow window) {
		for (Object s: selection.toArray()) {
			if (s instanceof FileTreeNodeFile) {
				FileTreeNodeFile	thisFile	= (FileTreeNodeFile) s;
				InputDialog			inputDialog	= new InputDialog(shell,
						messages.fileTreeContextMenuRenameTitle,
						messages.fileTreeContextMenuRenameDialog.replace("%1", ((FileTreeNodeFile) s).getName()),
						thisFile.getName(),
						newText -> newText.isEmpty() ? messages.fileTreeContextMenuRenameError : null);
				inputDialog.setBlockOnOpen(true);

				if (inputDialog.open() == Window.OK) {
					try {
						thisFile.moveTo(((FileTreeNodeFile) thisFile.getParent()).resolveChild(inputDialog.getValue()));

						thisFile.getParent().reload();
						((FileTreePart) filePart.getObject()).refresh(((FileTreeNode) s).getParent());
					} catch (Exception e) {
						LOGGER.error(window.hashCode(), e.getLocalizedMessage(), e);
					}
				}
			}
		}
	}

	@CanExecute
	public boolean canExecute(@Named(IServiceConstants.ACTIVE_SELECTION) ITreeSelection selection) {
		if (!selection.isEmpty()) {
			if (selection.getFirstElement() instanceof FileTreeNodeFile) {
				FileTreeNodeFile	firstFile	= (FileTreeNodeFile) selection.getFirstElement();
				try {
					return firstFile.canBeMovedTo(((FileTreeNodeFile) firstFile.getParent()).resolveChild("new_" + firstFile.getName()));
				} catch (Exception e) {
				}
			}
		}

		return false;
	}
}