package de.credo.lookup2.handlers.files;

import jakarta.inject.Inject;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledToolItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.workbench.modeling.EModelService;

import de.credo.lookup2.Constants;
import de.credo.lookup2.filetree.api.FileTree;
import de.credo.lookup2.filetree.api.FileTreeConstants;

public class ShowHiddenHandler {

	private	boolean	selected;

	@Execute
	public void execute(FileTree fileTree, IEventBroker eventBroker) {
		selected	= !selected;
		fileTree.setShowHiddenFiles(selected);
		eventBroker.send(FileTreeConstants.SET_SHOWHIDDEN_STATE, Boolean.valueOf(selected));	// Event anstatt direkter Aufruf der Funktion unten, da dieses Event alle offenen Fenster betrifft
		eventBroker.post(FileTreeConstants.REFRESH_FILE_TREE, fileTree);						// Den ganzen Baum neu laden
	}

	@Execute
	public void execute(MMenuItem menu, FileTree fileTree, IEventBroker eventBroker) {
		selected	= menu.isSelected();
		fileTree.setShowHiddenFiles(selected);
		eventBroker.send(FileTreeConstants.SET_SHOWHIDDEN_STATE, Boolean.valueOf(selected));	// Event anstatt direkter Aufruf der Funktion unten, da dieses Event alle offenen Fenster betrifft
		eventBroker.post(FileTreeConstants.REFRESH_FILE_TREE, fileTree);						// Den ganzen Baum neu laden
	}

	@CanExecute
	public boolean canExecute() {
		return true;
	}

	@Inject
	@Optional
	public void setSelection(@UIEventTopic(FileTreeConstants.SET_SHOWHIDDEN_STATE) Boolean selectedState, EModelService modelService, @Active MWindow window) {
		if (selectedState != null) {
			selected	= selectedState;
			selectButton(selectedState, window, modelService);
			selectMenu(selectedState, window, modelService);
		}
	}

	private void selectButton(boolean select, MWindow window, EModelService modelService) {
		MHandledToolItem	button	= (MHandledToolItem) modelService.find(Constants.SHOW_HIDDEN_BUTTON, window);
		selectButton(button, select);
	}

	private void selectButton(MHandledToolItem button, boolean select) {
		button.setIconURI(select ? button.getPersistedState().get(Constants.SELECTED_ICON_URI_KEY) : button.getPersistedState().get(Constants.UNSELECTED_ICON_URI_KEY));
	}

	private void selectMenu(boolean select, MWindow window, EModelService modelService) {
		MMenuItem	menu	= (MMenuItem) modelService.find(Constants.SHOWHIDDEN_MENU, window.getMainMenu());
		selectMenu(menu, select);
	}

	private void selectMenu(MMenuItem menu, boolean select) {
		menu.setSelected(select);
	}

}
