
package de.credo.lookup2.handlers.messages;

import java.util.BitSet;
import java.util.List;

import jakarta.inject.Inject;
import jakarta.inject.Named;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MHandledToolItem;
import org.eclipse.e4.ui.model.application.ui.menu.MMenuItem;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import de.credo.lookup2.Constants;
import de.credo.lookup2.parts.MessagePart;

public class MessageSelectionHandler {

	public final static String[]	MESSAGE_BUTTON_IDS	= new String[] {
			Constants.MESSAGE_LEVEL_BUTTON_VERBO,		// Ordinal 0
			Constants.MESSAGE_LEVEL_BUTTON_DEBUG,		// Ordinal 1
			Constants.MESSAGE_LEVEL_BUTTON_INFO,		// Ordinal 2
			Constants.MESSAGE_LEVEL_BUTTON_WARN,		// Ordinal 3
			Constants.MESSAGE_LEVEL_BUTTON_ERROR,		// Ordinal 4
			Constants.MESSAGE_LEVEL_BUTTON_FATAL};		// Ordinal 5

	private BitSet	messageBits	= new BitSet(6);		// verbose = 0, debug = 1, info = 2, warning = 3, error = 4, fatal = 5

	@Execute
	public void execute(MHandledToolItem handledItem, @Named(Constants.MESSAGE_SELECTION_BUTTON_ORDINAL) String buttonOrdinal, EPartService partService) {
		MessagePart	messagePart	= (MessagePart) partService.findPart(Constants.MESSAGE_PART).getObject();
		int severityOrdinal	= Integer.parseInt(buttonOrdinal);
		messageBits.flip(severityOrdinal);
		selectButton(handledItem, messageBits.get(severityOrdinal));
		messagePart.setMessageBits(messageBits);
	}

	@Execute
	public void execute(MMenuItem menu, EPartService partService) {
		MessagePart	messagePart	= (MessagePart) partService.findPart(Constants.MESSAGE_PART).getObject();
		messagePart.setMessageBits(messageBits);
	}

	@CanExecute
	public boolean canExecute() {
		return true;
	}

	@Inject
	@Optional
	public void setSelection(@UIEventTopic(Constants.ACTIVE_MESSAGE_LEVELS_EVENT) BitSet messages, EPartService partService) {
		if (messages != null) {
			messageBits	= messages;

			MPart					messagePart	= partService.findPart(Constants.MESSAGE_PART);
			List<MToolBarElement>	tbElements	= messagePart.getToolbar().getChildren();
			for (int i = 0; i < 6; i++) {
				for (MToolBarElement tbe: tbElements) {
					if (tbe.getElementId().equals(MESSAGE_BUTTON_IDS[i])) {
						selectButton((MHandledToolItem) tbe, messageBits.get(i));
						break;
					}
				}
			}
		}
	}

	private void selectButton(MHandledToolItem button, boolean select) {
		button.setIconURI(select ? button.getPersistedState().get(Constants.UNSELECTED_ICON_URI_KEY) : button.getPersistedState().get(Constants.DISABLED_ICON_URI_KEY));
	}
}