package de.credo.lookup2.handlers.structure;

import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.workbench.modeling.EPartService;

import de.credo.lookup2.Constants;
import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.parsers.api.structure.L2ParentElement;
import de.credo.lookup2.parsers.impl.RootElement;
import de.credo.lookup2.parts.structure.StructureTreePart;
import de.credo.lookup2.plugins.api.L2Constants;
import de.credo.lookup2.plugins.api.L2FileData;
import de.credo.lookup2.plugins.api.L2ProgressMonitor;

public class ReloadFileHandler {

	@Execute
	public void execute(GlobalAppManager globalAppManager, L2ProgressMonitor progressMonitor, L2FileData fileData, EPartService partService, MApplication application, @Active MTrimmedWindow window) {
		// Einschalten des Flags, dass man sich im Reload befindet. Das ist möglicherweise wichtig für die Viewer,
		// denen damit Gelegenheit gegeben wird, ihr (Neu-)Startverhalten an die Situation anzupassen. MultiView
		// beispielsweise verhindert in diesem Modus das Auslösen des PageNumberChanged-Events, das ansonsten un-
		// gewollte Auswirkungen auf Structure- und ObjectTree hat.
		application.getContext().set(L2Constants.RELOAD_MODE, Boolean.TRUE);

		StructureTreePart	structureTree	= (StructureTreePart) partService.findPart(Constants.STRUCTURE_TREE_PART).getObject();
		L2ParentElement		selection		= (L2ParentElement) structureTree.getSelection().getFirstElement();
		int[]				indexPath		= selection != null ? ((L2ParentElement) structureTree.getSelection().getFirstElement()).getTreePathIndex() : null;

		progressMonitor.setCanceled(true);
		globalAppManager.closeFile(window, false);
		globalAppManager.openInWindow(window, fileData, SwitchPerspectiveTargetTypes.NONE);

		if (indexPath != null) {
			RootElement		rootElement		= (RootElement) structureTree.getTreeViewer().getInput();
			new Timer().schedule(new TimerTask() {
				@Override
				public void run() {
					L2ParentElement	parentElement	= rootElement.getTreePathElement(indexPath);
					if (parentElement != null) {
						window.getContext().set(L2ParentElement.class, parentElement);		// Ruft in der Folge auch StructureTreePart#selectNode(...) auf!
						rootElement.getRootParser().selectionChanged(parentElement);		// Aktualisiert Viewer und ObjectsTree
						application.getContext().remove(L2Constants.RELOAD_MODE);
						cancel();
					}
				}
			}, 500, 500);
		}
	}

	@CanExecute
	public boolean canExecute(@Optional L2FileData fileData, @Optional RootElement rootElement) {
		return fileData != null && rootElement != null;
	}

}
