package de.credo.lookup2.parts;

import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;

import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.plugins.impl.FileData;

public class StartPart {

	private	final static	IPath	CREDO_BACKGROUND_LOGO_PATH	= new Path("src/main/resources/images/CRE-DO_Background.png");

	@Inject	GlobalAppManager	globalAppManager;
	@Inject	MTrimmedWindow		window;

	@PostConstruct
	public void postConstruct(Composite parent) {
		Composite	composite	= new Composite(parent, SWT.NONE);

		Bundle	bundle	= FrameworkUtil.getBundle(getClass());
		URL		imgFile	= FileLocator.find(bundle, CREDO_BACKGROUND_LOGO_PATH, null);
		composite.setBackgroundImage(ImageDescriptor.createFromURL(imgFile).createImage());

		composite.addDisposeListener(e -> composite.getBackgroundImage().dispose());

		DropTarget	target	= new DropTarget(composite, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
//		target.setTransfer(new Transfer[] {FileTransfer.getInstance(), URLTransfer.getInstance()});
		target.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		target.addDropListener(new DropTargetListener() {

			@Override	public void dropAccept(DropTargetEvent event) {}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data instanceof String[]) {
					String[]	filenames	= (String[]) event.data;
					if (filenames.length > 0) {
						for (int i = 0; i < filenames.length; i++) {
							java.nio.file.Path	filePath	= Paths.get(filenames[i]);
							if (i == 0) {
								// Erste Datei im aktuellen Fenster ...
								globalAppManager.openInWindow(window, new FileData(filePath, Files.isWritable(filePath)), SwitchPerspectiveTargetTypes.ANY);
							} else {
								// ... und weitere Dateien in weiteren, zusätzlichen Fenstern öffnen
								globalAppManager.openInNewWindow(new FileData(filePath, Files.isWritable(filePath)), true, SwitchPerspectiveTargetTypes.ANY);
							}
						}
					}
//				} else if (event.data instanceof String) {
//					try {
//						URL	url	= new URL((String) event.data);
//					} catch (MalformedURLException e) {
//					}
				}
			}

			@Override	public void dragOver(DropTargetEvent event) {}

			@Override	public void dragOperationChanged(DropTargetEvent event) {}

			@Override	public void dragLeave(DropTargetEvent event) {}

			@Override	public void dragEnter(DropTargetEvent event) {}
		});
	}

}