package de.credo.lookup2.parts;

import java.nio.file.Files;
import java.nio.file.Paths;

import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;

import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.workbench.UIEvents;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.osgi.service.event.EventHandler;

import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.Messages;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.ToolbarUtils;
import de.credo.lookup2.plugins.api.L2FileData;
import de.credo.lookup2.plugins.api.L2SearchCriteria;
import de.credo.lookup2.plugins.impl.FileData;
import de.credo.lookup2.plugins.impl.ModulesHandler;
import de.credo.lookup2.viewers.api.L2Searchable;
import de.credo.lookup2.viewers.api.L2ViewPart;
import de.credo.lookup2.viewers.api.L2Viewer;
import de.credo.lookup2.viewers.api.L2ViewerConfig;
import de.credo.lookup2.viewers.api.L2ViewerFactory;

public final class ViewPart implements L2ViewPart {

	@Inject	EventHandler		eventHandler;
	@Inject	IEventBroker		eventBroker;
	@Inject	UISynchronize		syncronizer;
	@Inject	MTrimmedWindow		window;
	@Inject	MPart				viewPart;
	@Inject	IEclipseContext		context;
	@Inject	GlobalAppManager	globalAppManager;
	@Inject	ModulesHandler		modulesHandler;
	@Optional
	@Inject	L2FileData			fileData;
	@Optional
	@Inject	L2ViewerConfig		config;
	@Inject	@Translation	Messages	messages;

	protected	Composite	parent;
	protected	L2Viewer	activeViewer;

	@PostConstruct
	public void postConstruct(Composite parent) {
		this.parent	= parent;

		parent.setLayout(new FillLayout());

		DropTarget	target	= new DropTarget(parent, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
//		target.setTransfer(new Transfer[] {FileTransfer.getInstance(), URLTransfer.getInstance()});
		target.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		target.addDropListener(new DropTargetListener() {

			@Override	public void dropAccept(DropTargetEvent event) {}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data instanceof String[]) {
					String[]	filenames	= (String[]) event.data;
					if (filenames.length > 0) {
						for (int i = 0; i < filenames.length; i++) {
							java.nio.file.Path	filePath	= Paths.get(filenames[i]);
							if (i == 0) {
								// Erste Datei im aktuellen Fenster ...
								globalAppManager.openInWindow(window, new FileData(filePath, Files.isWritable(filePath)), SwitchPerspectiveTargetTypes.ANY);
							} else {
								// ... und weitere Dateien in weiteren, zusätzlichen Fenstern öffnen
								globalAppManager.openInNewWindow(new FileData(filePath, Files.isWritable(filePath)), true, SwitchPerspectiveTargetTypes.ANY);
							}
						}
					}
//				} else if (event.data instanceof String) {
//					try {
//						URL	url	= new URL((String) event.data);
//					} catch (MalformedURLException e) {
//					}
				}
			}

			@Override	public void dragOver(DropTargetEvent event) {}

			@Override	public void dragOperationChanged(DropTargetEvent event) {}

			@Override	public void dragLeave(DropTargetEvent event) {}

			@Override	public void dragEnter(DropTargetEvent event) {}
		});

		// Händisches Nachholen der möglicherweise zu früh erfolgten Method Injection unten
		if (fileData != null) {
			openFile(fileData, config);
		}
	}

	@Focus
	public void setFocus() {
		parent.setFocus();

		if (activeViewer != null) {
			activeViewer.setFocus();

			ToolbarUtils.hideViewerToolbars(window);
			ToolbarUtils.showViewerToolbars(window, activeViewer);

			// TODO Nicht ausreichend! Eigentlich muss es eigene Routinen für Viewer- und Parser-Menüs geben
			ToolbarUtils.hideAllMenus(window);
			ToolbarUtils.showViewerMenus(window, activeViewer);
		}
	}

	@Inject
	public void openFile(@Optional L2FileData fileData, @Optional L2ViewerConfig config) {
		// Wurde dieser Part wegen "falscher" Method-Injection-Reihenfolge noch nicht initialisiert, dann wird
		// hier abgebrochen und die Übergabe lediglich zwischengespeichert. Die Routine wird dann später (nach
		// der Initialisierung) aus dem PostConstruct noch einmal neu aufgerufen.
		if (this.parent == null)
			return;

		if (fileData != null && fileData.getFile() != null && config != null) {
			if (config != null && config.creatable()) {
				L2ViewerFactory	factory = (L2ViewerFactory) modulesHandler.getFactory(config);				// Ermitteln der passenden Factory zu dieser Konfiguration

				if (activeViewer != null) {																	// Muss zuvor noch ein alter aktiver Viewer abgeschlossen werden?
					persist();
					closeFile();
					activeViewer.getFactory().destroyViewer(activeViewer);
				}

				activeViewer	= factory.createViewer(parent, config, window.getContext());
				activeViewer.setConfig(config);
				activeViewer.setPart(this);
				parent.layout();

				ToolbarUtils.hideViewerToolbars(window);
				ToolbarUtils.showViewerToolbars(window, activeViewer);

				ToolbarUtils.hideAllMenus(window);
				ToolbarUtils.showViewerMenus(window, activeViewer);

				// Händisches Auslösen der CanExecute-Abfrage in den Handlern. Diese wird seit Kepler nicht mehr als Timer ausgeführt, sondern
				// nur noch bei relevanten Änderungen an der Oberfläche, wie Partwechseln usw. Nötig zum Update von Toolbar-Buttons
				eventBroker.send(UIEvents.REQUEST_ENABLEMENT_UPDATE_TOPIC, UIEvents.ALL_ELEMENT_ID);

				syncronizer.asyncExec(() -> {
					activeViewer.openFile(fileData.getFile(), fileData.isWritable());
				});

				return;																			// Ende der erfolgreichen Verarbeitung!
			}
		}

		// Fallback im Problemfall, Löschen der Anzeige
		if (activeViewer != null) {																// Muss noch ein alter aktiver Viewer abgeschlossen werden?
			persist();
			closeFile();
			activeViewer.getFactory().destroyViewer(activeViewer);
		}

		viewPart.setLabel(messages.viewPartLabel);

		if (parent != null && !parent.isDisposed()) {
			activeViewer	= null;
			parent.layout();
		}

		ToolbarUtils.hideViewerToolbars(window);
		ToolbarUtils.hideAllMenus(window);
	}

	@Persist
	public void persist() {
		if (viewPart.isDirty()) {
			activeViewer.persist();
		}
	}

	public void closeFile() {
		activeViewer.closeFile();
	}

	@Override
	public L2Viewer getActiveViewer() {
		return activeViewer;
	}

	@Override
	public L2ViewPart getObject() {
		return this;
	}

	@Override
	public void setDirty(boolean changed) {
		viewPart.setDirty(changed);
	}

	@Override
	public boolean isDirty() {
		return viewPart.isDirty();
	}

	@Override
	public boolean canSearch() {
		return activeViewer != null && activeViewer instanceof L2Searchable && ((L2Searchable) activeViewer).canSearch();
	}

	@Override
	public void doSearch() {
		L2Searchable	searchable	= (L2Searchable) activeViewer;

		L2SearchCriteria	newCriteria	= searchable.showSearchDialog(context.get(L2SearchCriteria.class));
		if (newCriteria != null) {
			window.getContext().set(L2SearchCriteria.class, newCriteria);
			searchable.search(newCriteria);
		} else {
			window.getContext().remove(L2SearchCriteria.class);
		}
	}

	@Override
	public boolean canSearchAgain(L2SearchCriteria searchCriteria) {
		return activeViewer != null && activeViewer instanceof L2Searchable && ((L2Searchable) activeViewer).canSearchAgain(searchCriteria);
	}

	@Override
	public void doSearchAgain(L2SearchCriteria searchCriteria) {
		L2Searchable	searchable	= (L2Searchable) activeViewer;
		searchable.searchAgain(searchCriteria);
	}

}
