package de.credo.lookup2.parts.objects;

import java.nio.file.Files;
import java.nio.file.Paths;

import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import jakarta.inject.Inject;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.parsers.api.objects.L2DOMObject;
import de.credo.lookup2.plugins.impl.FileData;

public class ObjectsPropertiesPart {

	@Inject	UISynchronize		synchronizer;
	@Inject	Display				display;
	@Inject	GlobalAppManager	globalAppManager;
	@Optional
	@Inject	L2DOMObject			domObject;

	private	Composite	parent;
	private	Composite	composite;

	@PostConstruct
	public void postConstruct(Composite parent) {
		this.parent	= parent;

		createControls();

		// Händisches Nachholen der möglicherweise zu früh erfolgten Method Injection unten
		if (domObject != null) {
			printProperties(domObject);
		}
	}

	private void createControls() {
		composite	= new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout());
		composite.setBackground(display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));

		DropTarget	target	= new DropTarget(composite, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
		target.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		target.addDropListener(new DropTargetListener() {

			@Override	public void dropAccept(DropTargetEvent event) {}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data instanceof String[]) {
					String[]	filenames	= (String[]) event.data;
					if (filenames.length > 0) {
						for (int i = 0; i < filenames.length; i++) {
							java.nio.file.Path	filePath	= Paths.get(filenames[i]);
							globalAppManager.openInNewWindow(new FileData(filePath, Files.isWritable(filePath)), true, SwitchPerspectiveTargetTypes.ANY);
						}
					}
				}
			}

			@Override	public void dragOver(DropTargetEvent event) {}

			@Override	public void dragOperationChanged(DropTargetEvent event) {}

			@Override	public void dragLeave(DropTargetEvent event) {}

			@Override	public void dragEnter(DropTargetEvent event) {}
		});
	}

	@PreDestroy
	public void preDestroy() {

	}

	@Focus
	public void focus() {

	}

	@Persist
	public void persist() {

	}

	@Inject
	public void printProperties(@Optional L2DOMObject domObject) {
		// Wurde dieser Part wegen "falscher" Method-Injection-Reihenfolge noch nicht initialisiert, dann wird
		// hier abgebrochen und die Übergabe lediglich zwischengespeichert. Die Routine wird dann später (nach
		// der Initialisierung) aus dem PostConstruct noch einmal neu aufgerufen.
		if (this.parent == null)
			return;

		synchronizer.asyncExec(() -> {
			composite.dispose();

			if (domObject != null) {
				// Neuanlegen des eigenen Composite, entspricht einer Neuinitialisierung
				createControls();

				domObject.viewProperties(composite);

				composite.requestLayout();
			}
		});
	}
}