package de.credo.lookup2.parts.structure;

import java.nio.file.Files;
import java.nio.file.Paths;

import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import jakarta.inject.Inject;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.parsers.api.structure.L2ParentElement;
import de.credo.lookup2.plugins.impl.FileData;

public class StructurePropertiesPart {

	@Inject	UISynchronize		synchronizer;
	@Inject	Display				display;
	@Inject	GlobalAppManager	globalAppManager;
	@Optional
	@Inject	L2ParentElement		parentElement;

	private	Composite			parent;
	private	ScrolledComposite	scrollable;
	private	Composite			composite;

	@PostConstruct
	public void postConstruct(Composite parent) {
		this.parent	= parent;

		createControls();

		// Händisches Nachholen der möglicherweise zu früh erfolgten Method Injection unten
		if (parentElement != null) {
			printProperties(parentElement);
		}
	}

	private void createControls() {
		scrollable	= new ScrolledComposite(parent, SWT.V_SCROLL);
		scrollable.setAlwaysShowScrollBars(false);
		scrollable.setExpandHorizontal(true);
		scrollable.setExpandVertical(true);

		scrollable.addControlListener(new ControlListener() {

			@Override
			public void controlResized(ControlEvent e) {
				synchronizer.asyncExec(() -> {
					if (!composite.isDisposed()) {
						scrollable.setMinSize(composite.computeSize(scrollable.getClientArea().width, SWT.DEFAULT));
						scrollable.requestLayout();
					}
				});
			}

			@Override
			public void controlMoved(ControlEvent e) {
			}
		});

		composite	= new Composite(scrollable, SWT.NONE);
		RowLayout rowLayout = new RowLayout();
		rowLayout.type			= SWT.HORIZONTAL;
		rowLayout.wrap			= true;
//		rowLayout.pack			= true;
//		rowLayout.justify		= true;
		rowLayout.fill			= false;
		rowLayout.center		= true;
		rowLayout.marginLeft	= 10;
		rowLayout.marginTop		= 10;
		rowLayout.marginRight	= 10;
		rowLayout.marginBottom	= 10;
		rowLayout.marginHeight	= 10;
		rowLayout.marginWidth	= 10;
		rowLayout.spacing		= 0;
		composite.setLayout(rowLayout);

		composite.setBackground(display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));

		scrollable.setContent(composite);

		DropTarget	target	= new DropTarget(composite, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
		target.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		target.addDropListener(new DropTargetListener() {

			@Override	public void dropAccept(DropTargetEvent event) {}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data instanceof String[]) {
					String[]	filenames	= (String[]) event.data;
					if (filenames.length > 0) {
						for (int i = 0; i < filenames.length; i++) {
							java.nio.file.Path	filePath	= Paths.get(filenames[i]);
							globalAppManager.openInNewWindow(new FileData(filePath, Files.isWritable(filePath)), true, SwitchPerspectiveTargetTypes.ANY);
						}
					}
				}
			}

			@Override	public void dragOver(DropTargetEvent event) {}

			@Override	public void dragOperationChanged(DropTargetEvent event) {}

			@Override	public void dragLeave(DropTargetEvent event) {}

			@Override	public void dragEnter(DropTargetEvent event) {}
		});
	}

	@PreDestroy
	public void preDestroy() {

	}

	@Focus
	public void focus() {

	}

	@Persist
	public void persist() {

	}

	@Inject
	public void printProperties(@Optional L2ParentElement parentElement) {
		// Wurde dieser Part wegen "falscher" Method-Injection-Reihenfolge noch nicht initialisiert, dann wird
		// hier abgebrochen und die Übergabe lediglich zwischengespeichert. Die Routine wird dann später (nach
		// der Initialisierung) aus dem PostConstruct noch einmal neu aufgerufen.
		if (this.parent == null)
			return;

		synchronizer.asyncExec(() -> {
			scrollable.dispose();

			if (parentElement != null) {
				// Neuanlegen des eigenen Composite, entspricht einer Neuinitialisierung
				createControls();

				parentElement.viewProperties(composite);

				scrollable.requestLayout();
			}
		});
	}
}