package de.credo.lookup2.parts.structure;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import jakarta.inject.Inject;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.e4.core.contexts.Active;
import org.eclipse.e4.core.contexts.IEclipseContext;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MTrimmedWindow;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.DecoratingStyledCellLabelProvider;
import org.eclipse.jface.viewers.DecorationOverlayIcon;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

import de.credo.common.bundles.messages.CdoLogger;
import de.credo.lookup2.Constants;
import de.credo.lookup2.GlobalAppManager;
import de.credo.lookup2.Messages;
import de.credo.lookup2.SwitchPerspectiveTargetTypes;
import de.credo.lookup2.parsers.api.Marker;
import de.credo.lookup2.parsers.api.structure.L2Element;
import de.credo.lookup2.parsers.api.structure.L2Element.State;
import de.credo.lookup2.parsers.api.structure.L2ParentElement;
import de.credo.lookup2.parsers.api.structure.L2Parser;
import de.credo.lookup2.parsers.api.structure.L2RichLink;
import de.credo.lookup2.parsers.impl.RootElement;
import de.credo.lookup2.parsers.impl.StreamsHandler;
import de.credo.lookup2.parsers.impl.structure.StructureTreeSegment;
import de.credo.lookup2.plugins.api.L2Constants;
import de.credo.lookup2.plugins.api.L2Exception;
import de.credo.lookup2.plugins.api.L2SearchCriteria;
import de.credo.lookup2.plugins.impl.FileData;
import de.credo.lookup2.plugins.impl.ModulesHandler;
import de.credo.lookup2.plugins.impl.PerspectiveTypes;

public class StructureTreePart {

//	private final static	IEclipsePreferences	userPreferences	= InstanceScope.INSTANCE.getNode(StructureTreePart.class.getName());

	private final static	String	INFO_INDICATOR_DESCRIPTOR_KEY		= "StructureInfoIndicatorDescriptor";
	private final static	IPath	INFO_INDICATOR_DESCRIPTOR_PATH		= new Path("src/main/resources/icons/_16/infoIndicator.png");
	private final static	String	INFO_INDICATOR_IMAGE_KEY			= "StructureInfoIndicatorImage";
	private final static	IPath	INFO_INDICATOR_IMAGE_PATH			= new Path("src/main/resources/icons/_16/infoIcon.png");

	private final static	String	WARNING_INDICATOR_DESCRIPTOR_KEY	= "StructureWarningIndicatorDescriptor";
	private final static	IPath	WARNING_INDICATOR_DESCRIPTOR_PATH	= new Path("src/main/resources/icons/_16/warningIndicator.png");
	private final static	String	WARNING_INDICATOR_IMAGE_KEY			= "StructureWarningIndicatorImage";
	private final static	IPath	WARNING_INDICATOR_IMAGE_PATH		= new Path("src/main/resources/icons/_16/warningIcon.png");

	private final static	String	ERROR_INDICATOR_DESCRIPTOR_KEY		= "StructureErrorIndicatorDescriptor";
	private final static	IPath	ERROR_INDICATOR_DESCRIPTOR_PATH		= new Path("src/main/resources/icons/_16/errorIndicator.png");
	private final static	String	ERROR_INDICATOR_IMAGE_KEY			= "StructureErrorIndicatorImage";
	private final static	IPath	ERROR_INDICATOR_IMAGE_PATH			= new Path("src/main/resources/icons/_16/errorIcon.png");

	private final static	String	CHANGED_INDICATOR_DESCRIPTOR_KEY	= "StructureChangedIndicatorDescriptor";
	private final static	IPath	CHANGED_INDICATOR_DESCRIPTOR_PATH	= new Path("src/main/resources/icons/_16/changedIndicator.png");
	private final static	String	CHANGED_INDICATOR_IMAGE_KEY			= "StructureChangedIndicatorImage";
	private final static	IPath	CHANGED_INDICATOR_IMAGE_PATH		= new Path("src/main/resources/icons/_16/changedIcon.png");

	private	final static	CdoLogger	LOGGER	= new CdoLogger(L2Constants.LOGGER_ORIGIN_STRUCTURE);

	@Inject	UISynchronize		synchronizer;
	@Inject	IEventBroker		eventBroker;
	@Inject	ESelectionService	selectionService;
	@Inject	EMenuService		menuService;
	@Inject	MTrimmedWindow		window;
	@Inject	IEclipseContext		context;
	@Inject	MPart				structureTreePart;
	@Inject	EModelService		modelService;
	@Inject	GlobalAppManager	globalAppManager;
	@Inject	ModulesHandler		modulesHandler;
	@Inject	StreamsHandler		streamsHandler;
	@Optional
	@Inject	RootElement			rootElement;
	@Optional
	@Inject	L2ParentElement		parentElement;
	@Inject	@Translation	Messages	messages;

	private	Composite		parent;
	private	TreeViewer		treeViewer;
	private	boolean			rightMouseClicked;
	private	boolean			updateStartedHere;

	private	static	ImageRegistry	imageRegistry;

	static {
		Bundle	bundle	= FrameworkUtil.getBundle(StructureTreePart.class);
		imageRegistry	= JFaceResources.getImageRegistry();

		if (imageRegistry.get(INFO_INDICATOR_DESCRIPTOR_KEY) == null) {
			imageRegistry.put(INFO_INDICATOR_DESCRIPTOR_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, INFO_INDICATOR_DESCRIPTOR_PATH)));
		}
		if (imageRegistry.get(INFO_INDICATOR_IMAGE_KEY) == null) {
			imageRegistry.put(INFO_INDICATOR_IMAGE_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, INFO_INDICATOR_IMAGE_PATH)).createImage());
		}

		if (imageRegistry.get(WARNING_INDICATOR_DESCRIPTOR_KEY) == null) {
			imageRegistry.put(WARNING_INDICATOR_DESCRIPTOR_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, WARNING_INDICATOR_DESCRIPTOR_PATH)));
		}
		if (imageRegistry.get(WARNING_INDICATOR_IMAGE_KEY) == null) {
			imageRegistry.put(WARNING_INDICATOR_IMAGE_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, WARNING_INDICATOR_IMAGE_PATH)).createImage());
		}

		if (imageRegistry.get(ERROR_INDICATOR_DESCRIPTOR_KEY) == null) {
			imageRegistry.put(ERROR_INDICATOR_DESCRIPTOR_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, ERROR_INDICATOR_DESCRIPTOR_PATH)));
		}
		if (imageRegistry.get(ERROR_INDICATOR_IMAGE_KEY) == null) {
			imageRegistry.put(ERROR_INDICATOR_IMAGE_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, ERROR_INDICATOR_IMAGE_PATH)).createImage());
		}

		if (imageRegistry.get(CHANGED_INDICATOR_DESCRIPTOR_KEY) == null) {
			imageRegistry.put(CHANGED_INDICATOR_DESCRIPTOR_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, CHANGED_INDICATOR_DESCRIPTOR_PATH)));
		}
		if (imageRegistry.get(CHANGED_INDICATOR_IMAGE_KEY) == null) {
			imageRegistry.put(CHANGED_INDICATOR_IMAGE_KEY,
					ImageDescriptor.createFromURL(
							FileLocator.find(bundle, CHANGED_INDICATOR_IMAGE_PATH)).createImage());
		}
	}

	@PostConstruct
	public void postConstruct(Composite parent) {
		this.parent	= parent;

		createControls(parent);
		createContextMenu();

		loadPreferences();

		// Händisches Nachholen der möglicherweise zu früh erfolgten Method Injection unten
		if (rootElement != null) {
			reinitializeTree(rootElement);
		}

		// Händisches Nachholen der möglicherweise zu früh erfolgten Method Injection unten
		if (parentElement != null) {
			selectNode(parentElement);
		}
	}

	protected void createControls(Composite parent) {
		Composite	composite	= new Composite(parent, SWT.NONE);
		composite.setLayout(new FillLayout());

		treeViewer = new TreeViewer(composite, SWT.MULTI | SWT.V_SCROLL);

		treeViewer.getTree().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		treeViewer.getTree().setHeaderVisible(false);
		ColumnViewerToolTipSupport.enableFor(treeViewer);

		treeViewer.setContentProvider(new ITreeContentProvider() {

			@Override
			public Object[] getElements(Object input) {
				return ((RootElement) input).getTreeChildren().toArray();
			}

			@Override
			public Object getParent(Object element) {
				return ((L2ParentElement) element).getTreeParent();
			}

			@Override
			public Object[] getChildren(Object parentElement) {
				List<L2ParentElement>	childs	= ((L2ParentElement) parentElement).getTreeChildren();

				for (int i = 0; i < childs.size(); i++) {		// Klassische for-Schleife zur Vermeidung von ConcurrentModificationException
					L2Element	child	= childs.get(i);
					if (child.getState() == L2Element.State.UNKNOWN) {
						try {
							child.load();
						} catch (L2Exception e1) {
						}
					}
				}

				if (childs.size() > StructureTreeSegment.SEGMENT_SIZE) {
					List<L2ParentElement>	segments	= new ArrayList<>();
					for (int i = 0; i < childs.size(); i += StructureTreeSegment.SEGMENT_SIZE) {
						int	max	= Math.min(childs.size(), i + StructureTreeSegment.SEGMENT_SIZE);
						segments.add(new StructureTreeSegment(
								(L2ParentElement) parentElement,
								i + 1,
								max,
								childs.subList(i, max)));
					}
					return segments.toArray();

				} else
					return childs.toArray();
			}

			@Override
			public boolean hasChildren(Object parentElement) {
				return ((L2ParentElement) parentElement).hasTreeChildren();
			}
		});

		treeViewer.getTree().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
			}

			@Override
			public void mouseDown(MouseEvent e) {
				// Unterbinden des Selection-Events, wenn die rechte Maustaste gedrückt wurde. Kontextmenüs haben eh ihren eigenen MenuHandler
				rightMouseClicked	= e.button == 3;
			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
			}
		});

		treeViewer.addPostSelectionChangedListener(event -> {
			if (updateStartedHere)
				return;

			if ((ITreeSelection) event.getSelection() == null)
				return;

			if (((ITreeSelection) event.getSelection()).getFirstElement() == null)
				return;

			L2ParentElement	element	= (L2ParentElement) ((ITreeSelection) event.getSelection()).getFirstElement();

			if (element == null)
				return;

			try {
				if (element.getState() == State.UNKNOWN) {
					element.load();
				}

				// War ein Link-Element ausgewählt worden? Dann wird hier abgebrochen und erst einmal das verlinkte Objekt aufgerufen
				if (element instanceof L2RichLink) {
					window.getContext().set(L2ParentElement.class, ((L2RichLink<?>) element).getLinkTarget());
					return;
				}

				// Auswahl in den SelectionService, wo er von den Popup-Handlern für das Kontextmenü in Plugin.xml erwartet wird
				// und Ermitteln des Parents für die Popup-Core-Expressions
				selectionService.setSelection(event.getSelection());
				context.set("StructureSelectionParent", element.getParent());

				// Bis hier galt die Verarbeitung für jede Maustaste. Ab hier nur noch für die linke!
				if (rightMouseClicked)
					return;

				window.getContext().set(L2ParentElement.class, element);

				// Wurde ein Seitenobjekt ausgewählt, für das die Vorschau aktualisiert werden kann? Dann los! Das
				// aber natürlich nur, wenn das Baum-Update nicht selbst durch ein Viewer-Update ausgelöst worden war.
				element.getRootParser().selectionChanged(element);
			} catch (L2Exception e1) {
				// TODO: Warnung genauer spezifizieren
				LOGGER.fatal(window.hashCode(), e1.getLocalizedMessage(), e1);
			}
		});

		treeViewer.setLabelProvider(new DecoratingStyledCellLabelProvider(
				new IStyledLabelProvider() {

					@Override
					public StyledString getStyledText(Object element) {
						String			label			= ((L2ParentElement) element).getLabel();

						if (label == null)
							return new StyledString();

						StyledString	styledString	= new StyledString(label);

						if (((L2ParentElement) element).getDecorator() != null) {
							Styler		styler			= new Styler() {

								@Override
								public void applyStyles(TextStyle textStyle) {
									textStyle.background		= ((L2ParentElement) element).getDecorator().getBackground();

//									textStyle.borderColor		= null;
//									textStyle.borderStyle		= SWT.NONE;

									textStyle.foreground		= ((L2ParentElement) element).getDecorator().getForeground();

									textStyle.font				= ((L2ParentElement) element).getDecorator().getFont();

//									textStyle.rise				= 0;

									textStyle.strikeout			= ((L2ParentElement) element).getDecorator().strikeout();
//									textStyle.strikeoutColor	= null;

									textStyle.underline			= ((L2ParentElement) element).getDecorator().underline();
//									textStyle.underlineColor	= null;
//									textStyle.underlineStyle	= SWT.UNDERLINE_SINGLE;
								}
							};
							styledString.setStyle(0, label.length(), styler);
						}

						return styledString;
					}

					@Override
					public Image getImage(Object element) {
						return ((L2ParentElement) element).getImage();
					}

					@Override
					public void addListener(ILabelProviderListener listener) {
					}

					@Override
					public void dispose() {
					}

					@Override
					public boolean isLabelProperty(Object element, String property) {
						return false;
					}

					@Override
					public void removeListener(ILabelProviderListener listener) {
					}
				},
				new ILabelDecorator() {

					@Override
					public void removeListener(ILabelProviderListener listener) {
					}

					@Override
					public boolean isLabelProperty(Object element, String property) {
						return false;
					}

					@Override
					public void dispose() {
					}

					@Override
					public void addListener(ILabelProviderListener listener) {
					}

					@Override
					public String decorateText(String text, Object element) {
						return ((L2ParentElement) element).getDecorator() != null ? ((L2ParentElement) element).getDecorator().decorateText(text) : text;
					}

					@Override
					public Image decorateImage(Image image, Object element) {
						Image	decoratedImage	= ((L2ParentElement) element).getDecorator() != null ? ((L2ParentElement) element).getDecorator().decorateImage(image) : image;

						if (((L2ParentElement) element).getMarker().ordinal() > Marker.NONE.ordinal()) {
							if (decoratedImage == null) {
								switch (((L2ParentElement) element).getMarker()) {
								case INFO:
									return imageRegistry.get(INFO_INDICATOR_IMAGE_KEY);
								case WARNING:
									return imageRegistry.get(WARNING_INDICATOR_IMAGE_KEY);
								case ERROR:
									return imageRegistry.get(ERROR_INDICATOR_IMAGE_KEY);
								case CHANGED:
									return imageRegistry.get(CHANGED_INDICATOR_IMAGE_KEY);
								default:
								}

							} else {
								String	registryKey	= Integer.toHexString(decoratedImage.hashCode()) + "_" + ((L2ParentElement) element).getMarker().name();
								Image	overlay		= imageRegistry.get(registryKey);

								if (overlay == null) {
									ImageDescriptor	decoration	= null;

									switch (((L2ParentElement) element).getMarker()) {
									case INFO:
										decoration	= imageRegistry.getDescriptor(INFO_INDICATOR_DESCRIPTOR_KEY);
										break;
									case WARNING:
										decoration	= imageRegistry.getDescriptor(WARNING_INDICATOR_DESCRIPTOR_KEY);
										break;
									case ERROR:
										decoration	= imageRegistry.getDescriptor(ERROR_INDICATOR_DESCRIPTOR_KEY);
										break;
									case CHANGED:
										decoration	= imageRegistry.getDescriptor(CHANGED_INDICATOR_DESCRIPTOR_KEY);
										break;
									default:
									}

									overlay	= new DecorationOverlayIcon(decoratedImage, decoration, IDecoration.BOTTOM_LEFT).createImage();
									imageRegistry.put(registryKey, overlay);
								}

								return overlay;
							}
						}

						return decoratedImage;
					}
				},
				null) {

			@Override
			public String getToolTipText(Object element) {
				return ((L2ParentElement) element).getToolTipText();
			}

			@Override
			public boolean useNativeToolTip(Object object) {
				return true;
			}
		});

		DropTarget	target	= new DropTarget(composite, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
		target.setTransfer(new Transfer[] {FileTransfer.getInstance()});
		target.addDropListener(new DropTargetListener() {

			@Override	public void dropAccept(DropTargetEvent event) {}

			@Override
			public void drop(DropTargetEvent event) {
				if (event.data instanceof String[]) {
					String[]	filenames	= (String[]) event.data;
					if (filenames.length > 0) {
						for (int i = 0; i < filenames.length; i++) {
							java.nio.file.Path	filePath	= Paths.get(filenames[i]);
							globalAppManager.openInNewWindow(new FileData(filePath, Files.isWritable(filePath)), true, SwitchPerspectiveTargetTypes.ANY);
						}
					}
				}
			}

			@Override	public void dragOver(DropTargetEvent event) {}

			@Override	public void dragOperationChanged(DropTargetEvent event) {}

			@Override	public void dragLeave(DropTargetEvent event) {}

			@Override	public void dragEnter(DropTargetEvent event) {}
		});
	}

	private void createContextMenu() {
		menuService.registerContextMenu(getTreeViewer().getTree(), Constants.STRUCTURE_POPUP_MENU);
	}

	@Inject
	public void reinitializeTree(@Optional RootElement rootElement) {
		// Wurde dieser Part wegen "falscher" Method-Injection-Reihenfolge noch nicht initialisiert, dann wird
		// hier abgebrochen und die Übergabe lediglich zwischengespeichert. Die Routine wird dann später (nach
		// der Initialisierung) aus dem PostConstruct noch einmal neu aufgerufen.
		if (this.parent == null || rootElement == null)
			return;

		if (!treeViewer.getControl().isDisposed()) {
			treeViewer.setInput(rootElement);
		}
	}

	@Inject
	public void selectNodeFromViewer(@Optional @UIEventTopic(L2Constants.VIEWER_PROPERTIES_CHANGED_EVENT) Map<Class<?>, Object> values, @Active MTrimmedWindow window) {
		if (values == null || rootElement == null)
			return;

		Properties		viewerProperties	= (Properties) values.get(Properties.class);
		MTrimmedWindow	desiredWindow		= (MTrimmedWindow) values.get(MTrimmedWindow.class);

		// Neben der üblichen Abfrage, ob das an alle Fenster gesendete Event wirklich das gleiche Fenster des auslösenden
		// Viewers ist, muss hier auch noch die Perspektive passen, weil es die gleiche Routine auch noch einmal im
		// StructureTreePart gibt und ansonsten das Event ungewollt von beiden Perspektiven gleichzeitig, also doppelt,
		// ausgeführt wird.
		// Aber warum wird das Event überhaupt von beiden Perspektiven gefangen? Das hat mit dem Late Binding respektive
		// Laden der jeweiligen Perspektiven-Parts zu tun: Direkt nach dem Start der Anwendung kann es sein, dass die
		// weiteren, bislang unsichtbaren Parts in weiteren Perspektiven noch nie initialisiert wurden und damit auch
		// nicht für das Event Handling bereitstehen! Weil zur Entwicklungszeit nicht klar ist, ob der Benutzer sich
		// vorzugsweise in der Objekt- oder der Strukturansicht bewegt, muss das Event daher zur Sicherheit in beiden
		// Perspektiven gefangen und geprüft werden. Dennoch darf es hintendran natürlich nicht doppelt verarbeitet werden.
		if (window == desiredWindow && PerspectiveTypes.from(modelService.getActivePerspective(desiredWindow).getElementId()) == PerspectiveTypes.STRUCTURE) {
			L2Parser	parser	= rootElement.getRootParser();
			if (parser != null) {
				// Ruft in der Folge die untenstehende Routine selectNode() auf!
				parser.viewerChanged(viewerProperties);
			}
		}
	}

	@Inject
	public void selectNode(@Optional L2ParentElement element) {
		// Wurde dieser Part wegen "falscher" Method-Injection-Reihenfolge noch nicht initialisiert, dann wird
		// hier abgebrochen und die Übergabe lediglich zwischengespeichert. Die Routine wird dann später (nach
		// der Initialisierung) aus dem PostConstruct noch einmal neu aufgerufen.
		if (this.parent == null)
			return;

		if (element == null)
			return;

		if (element.getState() == State.UNKNOWN) {
			try {
				element.load();
			} catch (L2Exception e1) {
				// TODO: Warnung genauer spezifizieren
				LOGGER.fatal(window.hashCode(), e1.getLocalizedMessage(), e1);
			}
		}

		synchronizer.syncExec(() -> {
			updateStartedHere	= true;
			treeViewer.reveal(element);
			treeViewer.expandToLevel(element, 0);
			treeViewer.setSelection(new StructuredSelection(element), true);
			updateStartedHere	= false;
		});
	}

	@PreDestroy
	public void preDestroy() {
		// Einstellungen speichern. @Persist hilft hier nicht, weil der FileTree nie 'dirty' gesetzt wurde
		persistPreferences();
	}

	@Persist
	public void persist() {
		if (structureTreePart.isDirty()) {
//			structure.persist();
		}
	}

	private void loadPreferences() {
		// Initiales Befüllen der gerade neu erstellten Controls
	}

	private void persistPreferences() {
//		try {
//			userPreferences.flush();
//		} catch (BackingStoreException e) {
//		}
	}

	public ITreeSelection getSelection() {
		return treeViewer.getStructuredSelection();
	}

	public TreeViewer getTreeViewer() {
		return treeViewer;
	}

	public MWindow getWindow() {
		return window;
	}

	@Inject
	@Optional
	public void refreshTree(@UIEventTopic(L2Constants.REFRESH_STRUCTURE_TREE_EVENT) MTrimmedWindow desiredWindow, @Active MTrimmedWindow window) {
		// Dieses Event wird für alle Fenster gefeuert, obwohl die Aktion nur für ein spezielles gedacht ist
		if (window.equals(desiredWindow)) {
			refreshTree();
		}
	}

	public void refreshTree() {
		treeViewer.refresh();
	}

	@Inject
	@Optional
	public void refreshItem(@UIEventTopic(L2Constants.REFRESH_STRUCTURE_ITEM_EVENT) Map<Class<?>, Object> values, @Active MTrimmedWindow window) {
		if (values != null) {
			L2ParentElement	treeNode		= (L2ParentElement)	values.get(L2ParentElement.class);
			MTrimmedWindow	desiredWindow	= (MTrimmedWindow)	values.get(MTrimmedWindow.class);

			// Dieses Event wird für alle Fenster gefeuert, obwohl die Aktion nur für ein spezielles gedacht ist
			if (window.equals(desiredWindow)) {
				refreshItem(treeNode);
			}
		}
	}

	public void refreshItem(L2ParentElement treeNode) {
		treeViewer.refresh(treeNode, true);
	}

	public void doSearch() {
		L2SearchCriteria	newCriteria	= rootElement.getRootParser().showSearchStructureDialog(context.get(L2SearchCriteria.class));
		if (newCriteria != null) {
			window.getContext().set(L2SearchCriteria.class, newCriteria);

			switch (newCriteria.getSearchMode()) {
			case SEARCH:
				rootElement.getRootParser().searchStructure(newCriteria, getSelection().isEmpty()
						? rootElement.getRootParser()
						: (L2ParentElement) getSelection().getFirstElement());
				break;
			case COUNT:
				rootElement.getRootParser().searchStructure(newCriteria, rootElement.getRootParser());
				break;
			}
		} else {
			window.getContext().remove(L2SearchCriteria.class);
		}
	}

	public void doSearchAgain(L2SearchCriteria searchCriteria) {
		rootElement.getRootParser().searchStructureAgain(searchCriteria, getSelection().isEmpty()
				? rootElement
				: (L2ParentElement) getSelection().getFirstElement());
	}

}