package de.credo.lookup2.prefpages;

import java.net.URL;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.time.temporal.ChronoField;

import jakarta.inject.Inject;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;
import org.osgi.service.component.annotations.Component;

import de.credo.common.bundles.preferencepages.MyPrefPage;
import de.credo.common.bundles.version.BundleVersion;
import de.credo.lookup2.Messages;

@Component
public class AboutPreferencePage extends PreferencePage implements MyPrefPage {

	@Inject	Display			display;
	@Inject	@Translation	Messages	messages;

	private	final static	IPath	LOOKUP2_ICON_PATH	= new Path("splash.bmp");
	private	final static	DateTimeFormatter	dtf		= DateTimeFormatter.ofLocalizedDateTime(FormatStyle.LONG, FormatStyle.MEDIUM);

	private	final	Image	splashScreen;

	public AboutPreferencePage() {
		noDefaultAndApplyButton();

		Bundle	bundle	= FrameworkUtil.getBundle(getClass());
		URL		imgFile	= FileLocator.find(bundle, LOOKUP2_ICON_PATH, null);
		splashScreen	= ImageDescriptor.createFromURL(imgFile).createImage();
	}

	@Override
	public String getTitle() {
		return messages.aboutPreferencePageTitle;
	}

	@Override
	public int orderKey() {
		return 990;
	}

	@Override
	public String[] path() {
		return new String[] {messages.aboutPreferencePageTitle};
	}

	@Override
	public boolean creatable() {
		return true;
	}

	@Override
	protected Control createContents(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		Label	iconLabel	= new Label(container, SWT.NONE);
		iconLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true, 2, 1));
		iconLabel.setImage(splashScreen);

		Label	productName	= new Label(container, SWT.NONE);
		productName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 2, 1));
		productName.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
		productName.setText(messages.aboutPreferencePageProductName);

		Label	versionLabel	= new Label(container, SWT.NONE);
		versionLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, true));
		versionLabel.setText(messages.aboutPreferencePageVersion);

		Label	versionNumber	= new Label(container, SWT.NONE);
		versionNumber.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		versionNumber.setText(BundleVersion.getVersionNumber());

		Label	buildLabel	= new Label(container, SWT.NONE);
		buildLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, true));
		buildLabel.setText(messages.aboutPreferencePageBuildDate);

		Label	buildDate	= new Label(container, SWT.NONE);
		buildDate.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		buildDate.setText(dtf.format(BundleVersion.getBuildTime().atZone(ZoneId.of("Europe/Berlin"))));

		Label	publisherName	= new Label(container, SWT.NONE);
		publisherName.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, true));
		publisherName.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
		publisherName.setText(messages.aboutPreferencePagePublisher);

		Label	publisherCountry	= new Label(container, SWT.NONE);
		publisherCountry.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		publisherCountry.setText(messages.aboutPreferencePagePublisherCountry);

		Link	publisherDomain	= new Link(container, SWT.UNDERLINE_LINK | SWT.ITALIC);
		publisherDomain.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, true, 2, 1));
		publisherDomain.setText("<a>" + messages.aboutPreferencePagePublisherLink + "</a>");
		publisherDomain.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				Program.launch(messages.aboutPreferencePagePublisherLink);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		Link	productDomain	= new Link(container, SWT.UNDERLINE_LINK | SWT.ITALIC);
		productDomain.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 2, 1));
		productDomain.setText("<a>" + messages.aboutPreferencePageProductLink + "</a>");
		productDomain.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				Program.launch(messages.aboutPreferencePageProductLink);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		Label	copyright	= new Label(container, SWT.NONE);
		copyright.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 2, 1));
		copyright.setText("© " + messages.aboutPreferencePageCopyrightFrom + " - " + BundleVersion.getBuildTime().atZone(ZoneId.of("UTC")).get(ChronoField.YEAR));

		return container;
	}

	@Override
	public void dispose() {
		splashScreen.dispose();

		super.dispose();
	}
}
