package de.credo.lookup2.prefpages;

import jakarta.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.core.services.nls.Translation;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.osgi.service.component.annotations.Component;

import de.credo.common.bundles.preferencepages.MyPrefPage;
import de.credo.lookup2.Messages;
import de.credo.lookup2.filetree.api.FileTree;
import de.credo.lookup2.filetree.api.FileTreeConstants;

@Component
public class FileTreePreferencePage extends PreferencePage implements MyPrefPage {

	@Inject	IEventBroker	eventBroker;
	@Inject	FileTree		fileTree;
	@Inject	@Translation	Messages	messages;

	private	Button	showHiddenFilesCheckBox;
	private	Button	showSupportedOnlyCheckBox;
	private	Button	useQuickSelectionCheckBox;
	private	Button	autoColumnWidthCheckBox;
	private	Button	useWatchServiceCheckBox;
	private	Button	autoSwitchPerspectiveCheckBox;

//	public FileTreePreferencePage() {
//		noDefaultAndApplyButton();
//	}

	@Override
	public String getTitle() {
		return messages.fileTreeOptionsPreferencePageTitle;
	}

	@Override
	public int orderKey() {
		return 220;
	}

	@Override
	public String[] path() {
		return new String[] {messages.fileTreeOptionsPreferencePageTitle};
	}

	@Override
	public boolean creatable() {
		return true;
	}

	@Override
	protected Control createContents(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout pageLayout = new GridLayout(1, true);
		pageLayout.marginLeft = 0;
		pageLayout.marginRight = 0;
		pageLayout.marginTop = 0;
		pageLayout.marginBottom = 0;
		pageLayout.verticalSpacing = 20;
		pageLayout.horizontalSpacing = 0;
		composite.setLayout(pageLayout);

		createControls(composite);
		initGUI();

		return composite;
	}

	private void createControls(Composite composite) {
		Composite	fileTreeOptionsComposite = new Composite(composite, SWT.NONE);
		fileTreeOptionsComposite.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, true));
		GridLayout layout		= new GridLayout();
		layout.marginTop		= 10;
		layout.verticalSpacing	= 15;
		fileTreeOptionsComposite.setLayout(layout);

		showHiddenFilesCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		showHiddenFilesCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		showHiddenFilesCheckBox.setText(messages.fileTreeOptionsPreferencePageShowHiddenFiles);

		showSupportedOnlyCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		showSupportedOnlyCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		showSupportedOnlyCheckBox.setText(messages.fileTreeOptionsPreferencePageShowSupportedOnly);

		useQuickSelectionCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		useQuickSelectionCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		useQuickSelectionCheckBox.setText(messages.fileTreeOptionsPreferencePageUseQuickSelection);

		autoColumnWidthCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		autoColumnWidthCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		autoColumnWidthCheckBox.setText(messages.fileTreeOptionsPreferencePageAutoColumnWidth);

		useWatchServiceCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		useWatchServiceCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		useWatchServiceCheckBox.setText(messages.fileTreeOptionsPreferencePageUseWatchService);

		autoSwitchPerspectiveCheckBox	= new Button(fileTreeOptionsComposite, SWT.CHECK);
		autoSwitchPerspectiveCheckBox.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		autoSwitchPerspectiveCheckBox.setText(messages.fileTreeOptionsPreferencePageAutoSwitchPerspective);

		performDefaults();
	}

	private void initGUI() {
		showHiddenFilesCheckBox.setSelection(fileTree.isShowHiddenFiles());
		showSupportedOnlyCheckBox.setSelection(fileTree.isShowSupportedOnly());
		useQuickSelectionCheckBox.setSelection(fileTree.isUseQuickSelection());
		autoColumnWidthCheckBox.setSelection(fileTree.isAutoColumnWidth());
		useWatchServiceCheckBox.setSelection(fileTree.isUseWatchService());
		autoSwitchPerspectiveCheckBox.setSelection(fileTree.isAutoSwitchPerspective());
	}

	@Override
	protected void performDefaults() {
		super.performDefaults();

		showHiddenFilesCheckBox.setSelection(false);
		showSupportedOnlyCheckBox.setSelection(false);
		useQuickSelectionCheckBox.setSelection(true);
		autoColumnWidthCheckBox.setSelection(true);
		useWatchServiceCheckBox.setSelection(true);
		autoSwitchPerspectiveCheckBox.setSelection(false);
	}

	@Override
	public boolean performOk() {
		// Nur weitermachen, wenn diese Seite denn auch angezeigt worden war (= Objekte angelegt wurden)
		if (showSupportedOnlyCheckBox == null)
			return super.performOk();

		if (showHiddenFilesCheckBox.getSelection() != fileTree.isShowHiddenFiles()) {
			fileTree.setShowHiddenFiles(showHiddenFilesCheckBox.getSelection());
			eventBroker.send(FileTreeConstants.SET_SHOWHIDDEN_STATE, showHiddenFilesCheckBox.getSelection());		// Event anstatt direkter Aufruf der Funktion unten, da dieses Event alle offenen Fenster betrifft
		}
		if (showSupportedOnlyCheckBox.getSelection() != fileTree.isShowSupportedOnly()) {
			fileTree.setShowSupportedOnly(showSupportedOnlyCheckBox.getSelection());
			eventBroker.send(FileTreeConstants.SET_SHOWSUPPORTED_STATE, showSupportedOnlyCheckBox.getSelection());	// Event anstatt direkter Aufruf der Funktion unten, da dieses Event alle offenen Fenster betrifft
		}
		fileTree.setUseQuickSelection(useQuickSelectionCheckBox.getSelection());
		fileTree.setAutoColumnWidth(autoColumnWidthCheckBox.getSelection());
		fileTree.setUseWatchService(useWatchServiceCheckBox.getSelection());
		fileTree.setAutoSwitchPerspective(autoSwitchPerspectiveCheckBox.getSelection());

		eventBroker.post(FileTreeConstants.REFRESH_FILE_TREE, fileTree);											// Den ganzen Baum neu laden

		return super.performOk();
	}
}
