package de.credo.lookup2.prefpages;

import jakarta.inject.Inject;
import jakarta.inject.Singleton;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UIEventTopic;
import org.eclipse.swt.widgets.Shell;

import de.credo.common.bundles.preferencepages.AbstractPrefPageHandler;
import de.credo.common.bundles.preferencepages.MyPrefPage;
import de.credo.lookup2.plugins.api.L2Constants;

/**
 * OrderKey-Gruppen für eine Vorsortierung der Preference Pages im Falle von Lookup2:
 * 100-199: Allgemeines (1)
 *     110: Starteinstellungen
 * 200-499: Programmfunktionen
 *     210: Supported file formats
 *     220: File Tree Settings
 *     221: File Tree Settings.Supported Archive formats
 *     222: File Tree Settings.Targets
 * 500-699: Modulfunktionen
 *     600: MultiView
 *     610: MultiView.AFP-specific
 *     611: MultiView.AFP-specific Resource Library
 *     612: MultiView.AFP-specific Tray settings
 *     690: MultiView.Print Settings
 * 700-899: Format-PrefPages
 *     710: AFP
 *     711: AFP.Codepages
 *     712: AFP.PageGrouping
 *     750: XML
 *     740: Text
 * 900-999: Allgemeines (1)
 *     920: Proxy
 *     930: License
 *     932: License.Server Settings
 *     934: License.Features
 *     990: About
 */
@Creatable @Singleton
public class MyPrefPageHandler extends AbstractPrefPageHandler {

	@Override
	protected boolean expandAll() {
		return true;
	}

	@Inject
	@Optional
	public void openPreferencePage(@UIEventTopic(L2Constants.OPEN_PREF_PAGE_EVENT) String pageToOpenClassSimpleName, Shell shell) {
		if (pageToOpenClassSimpleName != null) {
			showDialog(shell, pageToOpenClassSimpleName);
		}
	}

	@Inject
	@Optional
	public void openPreferencePage(@UIEventTopic(L2Constants.OPEN_PREF_PAGE_EVENT) Class<? extends MyPrefPage> pageToOpen, Shell shell) {
		if (pageToOpen != null) {
			showDialog(shell, pageToOpen);
		}
	}

}
